'''`bestdori.logincampaigns`

BanG Dream! 登录奖励相关操作'''
from typing import Any, Dict, List, Literal

from aiohttp import ClientResponseError
from httpx import Response, HTTPStatusError

from .utils.utils import API, ASSETS
from .utils.network import Api, Assets
from .post import get_list, get_list_async
from .exceptions import (
    NoDataException,
    ServerNotAvailableError,
    LoginCampaignNotExistError
)

# 获取总登录奖励信息
def get_all(index: Literal[0, 5]=5) -> Dict[str, Dict[str, Any]]:
    '''获取总登录奖励信息

    参数:
        index (Literal[0, 5], optional): 指定获取哪种 `all.json`
            `0`: 仅获取所有已有登录奖励 ID `all.0.json`
            `5`: 获取所有已有登录奖励的简洁信息 `all.5.json`

    返回:
        Dict[str, Dict[str, Any]]: 获取到的总登录奖励信息
    '''
    return Api(API['loginCampaigns']['all'].format(index=index)).get().json()

# 异步获取总登录奖励信息
async def get_all_async(index: Literal[0, 5]=5) -> Dict[str, Dict[str, Any]]:
    '''获取总登录奖励信息

    参数:
        index (Literal[0, 5], optional): 指定获取哪种 `all.json`
            `0`: 仅获取所有已有登录奖励 ID `all.0.json`
            `5`: 获取所有已有登录奖励的简洁信息 `all.5.json`

    返回:
        Dict[str, Dict[str, Any]]: 获取到的总登录奖励信息
    '''
    response = await Api(API['loginCampaigns']['all'].format(index=index)).aget()
    if isinstance(response, Response): return response.json()
    return await response.json()

# 登录奖励类
class LoginCampaign:
    '''登录奖励类

    参数:
        id (int): 登录奖励 ID
    '''
    # 初始化
    def __init__(self, id: int) -> None:
        '''登录奖励类

        参数:
            id (int): 登录奖励 ID
        '''
        self.id: int = id
        '''登录奖励 ID'''
        self.__info: Dict[str, Any] = {}
        '''登录奖励信息'''
        return
    
    # 登录奖励标题
    @property
    def name(self) -> str:
        '''登录奖励标题'''
        info = self.__info
        # 获取 eventName 数据
        if (caption := info.get('caption', None)) is None:
            raise NoDataException('登录奖励标题')
        # 获取第一个非 None 登录奖励标题
        try:
            return next(filter(lambda x: x is not None, caption))
        except StopIteration:
            raise NoDataException('登录奖励标题')
    
    # 登录奖励默认服务器
    @property
    def server(self) -> Literal['jp', 'en', 'tw', 'cn', 'kr']:
        '''登录奖励默认服务器'''
        info = self.__info
        # 获取 startAt 数据
        if (published_at := info.get('publishedAt', None)) is None:
            raise NoDataException('登录奖励起始时间')
        # 根据 startAt 数据判断服务器
        if published_at[0] is not None: return 'jp'
        elif published_at[1] is not None: return 'en'
        elif published_at[2] is not None: return 'tw'
        elif published_at[3] is not None: return 'cn'
        elif published_at[4] is not None: return 'kr'
        else:
            raise NoDataException('登录奖励所在服务器')
    
    # 获取登录奖励信息
    def get_info(self) -> Dict[str, Any]:
        '''获取登录奖励信息

        返回:
            Dict[str, Any]: 登录奖励详细信息
        '''
        try:
            response = Api(
                API['loginCampaigns']['info'].format(id=self.id)
            ).get()
        except HTTPStatusError as exception:
            if exception.response.status_code == 404:
                raise LoginCampaignNotExistError(self.id)
            raise exception
        
        self.__info = dict(response.json())
        return self.__info
    
    # 异步获取登录奖励信息
    async def get_info_async(self) -> Dict[str, Any]:
        '''获取登录奖励信息

        返回:
            Dict[str, Any]: 登录奖励详细信息
        '''
        try:
            response = await Api(
                API['loginCampaigns']['info'].format(id=self.id)
            ).aget()
        except HTTPStatusError as exception:
            if exception.response.status_code == 404:
                raise LoginCampaignNotExistError(self.id)
            raise exception
        except ClientResponseError as exception:
            if exception.status == 404:
                raise LoginCampaignNotExistError(self.id)
            raise exception
        
        if isinstance(response, Response):
            self.__info = dict(response.json())
        else:
            self.__info = dict(await response.json())
        return self.__info
    
    # 获取缓存信息
    def __get_info_cache(self) -> Dict[str, Any]:
        '''获取缓存信息

        返回:
            Dict[str, Any]: 缓存信息
        '''
        if not self.__info:
            return self.get_info()
        return self.__info
    
    # 异步获取缓存信息
    async def __get_info_cache_async(self) -> Dict[str, Any]:
        '''获取缓存信息

        返回:
            Dict[str, Any]: 缓存信息
        '''
        if not self.__info:
            return await self.get_info_async()
        return self.__info
    
    # 获取登录奖励评论
    def get_comment(
        self,
        limit: int=20,
        offset: int=0,
        order: Literal['TIME_DESC', 'TIME_ASC']='TIME_ASC'
    ) -> Dict[str, Any]:
        '''获取登录奖励评论

        参数:
            limit (int, optional): 展示出的评论数，默认为 20
            offset (int, optional): 忽略前面的 `offset` 条评论，默认为 0
            order (Literal[&#39;TIME_DESC&#39;, &#39;TIME_ASC&#39;], optional): 排序顺序，默认时间顺序

        返回:
            Dict[str, Any]: 搜索结果
                ```python
                {
                    "result": ... # bool 是否有响应
                    "count": ... # int 搜索到的评论总数
                    "posts": ... # List[Dict[str, Any]] 列举出的评论
                }
                ```
        '''
        return get_list(
            category_name='LOGINCAMPAIGN_COMMENT',
            category_id=str(self.id),
            order=order,
            limit=limit,
            offset=offset
        )
    
    # 异步获取登录奖励评论
    async def get_comment_async(
        self,
        limit: int=20,
        offset: int=0,
        order: Literal['TIME_DESC', 'TIME_ASC']='TIME_ASC'
    ) -> Dict[str, Any]:
        '''获取登录奖励评论

        参数:
            limit (int, optional): 展示出的评论数，默认为 20
            offset (int, optional): 忽略前面的 `offset` 条评论，默认为 0
            order (Literal[&#39;TIME_DESC&#39;, &#39;TIME_ASC&#39;], optional): 排序顺序，默认时间顺序

        返回:
            Dict[str, Any]: 搜索结果
                ```python
                {
                    "result": ... # bool 是否有响应
                    "count": ... # int 搜索到的评论总数
                    "posts": ... # List[Dict[str, Any]] 列举出的评论
                }
                ```
        '''
        return await get_list_async(
            category_name='LOGINCAMPAIGN_COMMENT',
            category_id=str(self.id),
            order=order,
            limit=limit,
            offset=offset
        )
    
    # 获取登录奖励背景图图像
    def get_background(self, server: Literal['jp', 'en', 'tw', 'cn', 'kr']) -> bytes:
        '''获取登录奖励背景图图像

        参数:
            server (Literal[&#39;jp&#39;, &#39;en&#39;, &#39;tw&#39;, &#39;cn&#39;, &#39;kr&#39;]): 指定服务器

        返回:
            bytes: 登录奖励背景图图像字节数据 `bytes`
        '''
        # 获取登录奖励数据包名称
        info = self.__get_info_cache()
        if (asset_bundle_name := info.get('assetBundleName', None)) is None:
            raise ValueError('无法获取登录奖励数据包名称。')
        # 判断服务器
        SERVERS = ['jp', 'en', 'tw', 'cn', 'kr']
        index = SERVERS.index(server)
        if asset_bundle_name[index] is None:
            raise ServerNotAvailableError(f'登录奖励 {self.name}', server)
        return Assets(
            ASSETS['event']['loginbouns'].format(
                asset_bundle_name=asset_bundle_name[index]
            ), server
        ).get()
    
    # 异步获取登录奖励背景图图像
    async def get_background_async(self, server: Literal['jp', 'en', 'tw', 'cn', 'kr']) -> bytes:
        '''获取登录奖励背景图图像

        参数:
            server (Literal[&#39;jp&#39;, &#39;en&#39;, &#39;tw&#39;, &#39;cn&#39;, &#39;kr&#39;]): 指定服务器

        返回:
            bytes: 登录奖励背景图图像字节数据 `bytes`
        '''
        # 获取登录奖励数据包名称
        info = await self.__get_info_cache_async()
        if (asset_bundle_name := info.get('assetBundleName', None)) is None:
            raise ValueError('无法获取登录奖励数据包名称。')
        # 判断服务器
        SERVERS = ['jp', 'en', 'tw', 'cn', 'kr']
        index = SERVERS.index(server)
        if asset_bundle_name[index] is None:
            raise ServerNotAvailableError(f'登录奖励 {self.name}', server)
        return await Assets(
            ASSETS['event']['loginbouns'].format(
                asset_bundle_name=asset_bundle_name[index]
            ), server
        ).aget()



'''`bestdori.exceptions`

API 错误信息相关操作'''
from typing import Any, Dict

from httpx._exceptions import RequestError

# 错误基类
class BestdoriException(Exception):
    '''错误基类'''
    # 初始化
    def __init__(self, msg: str) -> None:
        self.message = msg
        '''错误信息'''
    
    # 字符串化
    def __str__(self) -> str:
        '''输出字符串'''
        return self.message

# 请求发送错误
class RequestException(BestdoriException):
    '''请求发送错误'''
    # 初始化
    def __init__(self, api: str, msg: str='无错误代码获取', **kwargs: Any) -> None:
        if len(kwargs) > 0:
            msg += f': {kwargs}'
        else:
            msg += '。'
        super().__init__(msg)
        self.api = api
        '''请求所使用的 API'''
    
    # 字符串化
    def __str__(self) -> str:
        '''输出字符串'''
        return f'向 Bestdori {self.api} 发送请求时出错。{self.message}'

# 资源错误
class AssetsException(BestdoriException):
    '''资源错误'''
    # 初始化
    def __init__(self, msg: str, **kwargs: Any) -> None:
        if len(kwargs) > 0:
            msg += f': {kwargs}'
        else:
            msg += '。'
        super().__init__(msg)
    
    # 字符串化
    def __str__(self) -> str:
        '''输出字符串'''
        return f'获取资源时出错。{self.message}'

# 帖子错误
class PostException(BestdoriException):
    '''帖子错误'''
    # 初始化
    def __init__(self, msg: str) -> None:
        super().__init__(msg)
        self.message = msg
        '''错误信息'''
        return

# 帖子不是社区谱面
class PostHasNoChartError(PostException):
    '''帖子不是社区谱面'''
    # 初始化
    def __init__(self, post: Dict[str, Any]) -> None:
        name = post.get('categoryName', 'DEFAULT_POST')
        msg = f'该帖子类型 {name} 不是社区谱面。'
        super().__init__(msg)
        return

# 帖子没有音乐字段
class PostHasNoSongError(PostException):
    '''帖子没有音乐字段'''
    # 初始化
    def __init__(self, post: Dict[str, Any]) -> None:
        name = post.get('categoryName', 'DEFAULT_POST')
        msg = f'该帖子类型 {name} 不存在歌曲资源。'
        super().__init__(msg)
        return

# 某 id 指定的资源不存在
class NotExistException(BestdoriException):
    '''资源不存在'''
    # 初始化
    def __init__(self, src: str) -> None:
        msg = f'{src} 不存在。'
        super().__init__(msg)
        self.message = msg
        '''错误信息'''
        return

# 歌曲不存在
class SongNotExistError(NotExistException):
    '''歌曲不存在'''
    # 初始化
    def __init__(self, id: int) -> None:
        super().__init__(f'歌曲 ID {id}')
        return

# 歌曲难度不存在
class DiffNotExistError(NotExistException):
    '''歌曲难度不存在'''
    # 初始化
    def __init__(self, diff: str) -> None:
        msg = f'歌曲难度 {diff}'
        super().__init__(msg)
        return

# 请求无效
class RequestInvalidError(RequestException):
    '''请求无效'''
    # 初始化
    def __init__(self, api: str, **kwargs: Any) -> None:
        msg = '请求无效'
        super().__init__(api, msg, **kwargs)
        return

# 需要登录
class LoginRequiredError(RequestException):
    '''需要登录'''
    # 初始化
    def __init__(self, api: str, **kwargs: Any) -> None:
        msg = '需要登录'
        super().__init__(api, msg, **kwargs)
        return

# 证书无效
class CredentialsInvalidError(RequestException):
    '''证书无效'''
    # 初始化
    def __init__(self, api: str, **kwargs: Any) -> None:
        msg = '证书无效'
        super().__init__(api, msg, **kwargs)
        return

# 用户无效
class UserInvalidError(RequestException):
    '''用户无效'''
    # 初始化
    def __init__(self, api: str, **kwargs: Any) -> None:
        msg = '用户无效'
        super().__init__(api, msg, **kwargs)
        return

# 文件已经被上传过
class AlreadyUploadedError(RequestException):
    '''文件已经被上传过'''
    # 初始化
    def __init__(self, api: str, **kwargs: Any) -> None:
        msg = '文件已经被上传过'
        super().__init__(api, msg, **kwargs)
        return

# 帖子无效
class PostInvalidError(RequestException):
    '''帖子无效'''
    # 初始化
    def __init__(self, api: str, **kwargs: Any) -> None:
        msg = '帖子无效'
        super().__init__(api, msg, **kwargs)
        return

# 资源不存在
class AssetsNotExistError(AssetsException):
    '''资源不存在'''
    # 初始化
    def __init__(self, asset_name: str) -> None:
        msg = f'资源 {asset_name} 可能不存在。'
        super().__init__(msg)

# 角色不存在
class CharacterNotExistError(NotExistException):
    '''角色不存在'''
    # 初始化
    def __init__(self, id: int) -> None:
        msg = f'角色 ID {id}'
        super().__init__(msg)
        return

# 卡牌不存在
class CardNotExistError(NotExistException):
    '''卡牌不存在'''
    # 初始化
    def __init__(self, id: int) -> None:
        msg = f'卡牌 ID {id}'
        super().__init__(msg)
        return

# 服装不存在
class CostumeNotExistError(NotExistException):
    '''服装不存在'''
    # 初始化
    def __init__(self, id: int) -> None:
        msg = f'服装 ID {id}'
        super().__init__(msg)
        return

# 活动不存在
class EventNotExistError(NotExistException):
    '''活动不存在'''
    # 初始化
    def __init__(self, id: int) -> None:
        msg = f'活动 ID {id}'
        super().__init__(msg)
        return

# 招募不存在
class GachaNotExistError(NotExistException):
    '''招募不存在'''
    # 初始化
    def __init__(self, id: int) -> None:
        msg = f'招募 ID {id}NotExistException'
        super().__init__(msg)
        return

# 登录奖励不存在
class LoginCampaignNotExistError(NotExistException):
    '''登录奖励不存在'''
    # 初始化
    def __init__(self, id: int) -> None:
        msg = f'登录奖励 ID {id} 不存在。'
        super().__init__(msg)
        return

# 自选券不存在
class MiracleTicketExchangeNotExistError(NotExistException):
    '''自选券不存在'''
    # 初始化
    def __init__(self, id: int) -> None:
        msg = f'自选券 ID {id} 不存在。'
        super().__init__(msg)
        return

# 漫画不存在
class ComicNotExistError(NotExistException):
    '''漫画不存在'''
    # 初始化
    def __init__(self, id: int) -> None:
        msg = f'漫画 ID {id} 不存在。'
        super().__init__(msg)
        return

# 任务不存在
class MissionNotExistError(NotExistException):
    '''任务不存在'''
    # 初始化
    def __init__(self, id: int) -> None:
        msg = f'任务 ID {id} 不存在。'
        super().__init__(msg)
        return

# 玩家不存在
class PlayerNotExistError(NotExistException):
    '''玩家不存在'''
    # 初始化
    def __init__(self, server: str, id: int) -> None:
        msg = f'服务器 {server} 上的玩家 ID {id} 不存在。'
        super().__init__(msg)
        return

# 服务器指定错误
class ServerNotAvailableError(BestdoriException):
    '''服务器指定错误'''
    # 初始化
    def __init__(self, name: str, server: str) -> None:
        msg = f'{name} 在服务器 {server} 不可用。'
        super().__init__(msg)
        self.message = msg
        '''错误信息'''
        return

# 活动没有奖励贴纸错误
class EventHasNoStampError(BestdoriException):
    '''活动没有奖励贴纸错误'''
    # 初始化
    def __init__(self, id: int) -> None:
        msg = f'活动 ID {id} 没有奖励贴纸。'
        super().__init__(msg)
        self.message = msg
        '''错误信息'''
        return

# 无法获取到信息错误
class NoDataException(BestdoriException):
    '''无法获取到信息错误'''
    # 初始化
    def __init__(self, src: str) -> None:
        msg = f'无法获取 {src} 。'
        super().__init__(msg)
        self.message = msg
        '''错误信息'''
        return

# 设置出错
class SettingsException(BestdoriException):
    '''设置出错'''
    # 初始化
    def __init__(self, msg: str) -> None:
        super().__init__(msg)
        self.message = msg
        '''错误信息'''
        return

# 没有设置 Cookies
class NoCookiesError(SettingsException):
    '''没有设置 Cookies'''
    # 初始化
    def __init__(self) -> None:
        msg = '没有设置 Cookies'
        super().__init__(msg)
        return

# 没有获取到 ContentType
class NoContentTypeError(RequestError):
    '''没有获取到 ContentType'''
    # 初始化
    def __init__(self, url: str) -> None:
        msg = f'没有获取到 {url} 的 ContentType'
        super().__init__(msg)
        return

# 请求错误集合
REQUEST_EXCEPTION: Dict[str, type[RequestException]] = {
    'REQUEST_INVALID': RequestInvalidError,
    'LOGIN_REQUIRED': LoginRequiredError,
    'CREDENTIALS_INVALID': CredentialsInvalidError,
    'USER_INVALID': UserInvalidError,
    'ALREADY_UPLOADED': AlreadyUploadedError,
    'POST_INVALID': PostInvalidError
}
'''请求错误集合'''


'''`bestdori.post`

社区帖子相关操作'''
from typing_extensions import overload
from typing import (
    TYPE_CHECKING,
    Any,
    Dict,
    List,
    Union,
    Literal,
    Optional,
    TypedDict
)

from httpx import Response

from .charts import Chart
from .models.content import Content
from .utils.utils import API, ASSETS
from .utils.network import Api, Assets
from .exceptions import (
    NoDataException,
    RequestException,
    PostHasNoSongError,
    AssetsNotExistError,
    PostHasNoChartError
)

if TYPE_CHECKING:
    from .user import Me

# 标签类
class Tag(TypedDict):
    '''标签类'''
    type: str
    '''标签类型'''
    data: str
    '''标签数据'''

# 歌曲资源类
class SongRes(TypedDict):
    '''歌曲资源类'''
    audio: Union[bytes, None]
    '''音频字节'''
    cover: Union[bytes, None]
    '''封面字节'''

# 自定义歌曲信息类
class CustomSong(TypedDict):
    '''自定义歌曲信息类'''
    type: Literal['custom']
    '''歌曲类型'''
    audio: Optional[str]
    '''歌曲音频'''
    cover: Optional[str]
    '''歌曲封面'''

# 服务器歌曲信息类
class ProvidedSong(TypedDict):
    '''服务器歌曲信息类'''
    type: Literal['bandori', 'llsif']
    '''歌曲类型'''
    id: int
    '''歌曲 ID'''

# 搜索社区谱面
@overload
def get_list(
    *,
    search: str='',
    category_name: Literal['SELF_POST']='SELF_POST',
    category_id: Literal['chart']='chart',
    tags: List[Tag]=[],
    order: Literal['TIME_DESC', 'TIME_ASC']='TIME_DESC',
    limit: int=20,
    offset: int=0
) -> Dict[str, Any]:
    '''搜索社区谱面
        ```python
        # 以 'Arghena' 为关键词，搜索社区谱面
        Post.search(search='Arghena', caregory_name='SELF_POST', category_id='chart')
        ```

    参数:
        search (str, optional): 搜索关键词，默认为空
        category_name (Literal[&#39;SELF_POST&#39;], optional): 搜索的帖子类型 `SELF_POST`
        category_id (Literal[&#39;chart&#39;, &#39;text&#39;], optional): 搜索的画廊种类 `chart`
        tags (List[Tag], optional): 搜索的标签，默认为空
        order (Literal[&#39;TIME_DESC&#39;, &#39;TIME_ASC&#39;], optional): 帖子排序，默认时间倒序
        limit (int, optional): 展示出的帖子数，默认 20
        offset (int, optional): 忽略前面的 `offset` 个帖子，默认 0

    返回:
        Dict[str, Any]: 搜索结果
            ```python
            result: bool # 是否有响应
            count: int # 搜索到的谱面总数
            posts: List[Dict[str, Any]] # 列举出的谱面
            ```
    '''
    ...

# 搜索用户帖子
@overload
def get_list(
    *,
    username: str,
    limit: int=20,
    offset: int=0,
    order: Literal['TIME_DESC', 'TIME_ASC']='TIME_DESC'
) -> Dict[str, Any]:
    '''搜索用户帖子

    参数:        
        username (str): 用户名
        limit (int, optional): 展示出的帖子数，默认 20
        offset (int, optional): 忽略前面的 `offset` 个帖子，默认 0
        order (Literal[&#39;TIME_DESC&#39;, &#39;TIME_ASC&#39;], optional): 帖子排序，默认时间倒序

    返回:
        Dict[str, Any]: 搜索结果
            ```python
            result: bool # 是否有响应
            count: int # 搜索到的帖子总数
            posts: List[Dict[str, Any]] # 列举出的帖子
            ```
    '''
    ...

# 搜索帖子
@overload
def get_list(
    *,
    search: Optional[str]=None,
    following: Optional[bool]=None,
    category_name: Optional[str]=None,
    category_id: Optional[str]=None,
    tags: Optional[List[Tag]]=None,
    username: Optional[str]=None,
    order: Literal['TIME_DESC', 'TIME_ASC'],
    limit: int=20,
    offset: int=0
) -> Dict[str, Any]:
    '''搜索帖子

    参数:        
        order (Literal[&#39;TIME_DESC&#39;, &#39;TIME_ASC&#39;]): 帖子排序
        search (Optional[str], optional): 搜索关键词
        following (Optional[bool], optional): 是否关注
        category_name (Optional[str], optional): 画廊名称
        category_id (Optional[str], optional): 画廊 ID
        tags (Optional[List[Tag]], optional): 帖子标签
        username (Optional[str], optional): 用户名
        limit (int, optional): 展示出的帖子数，默认 20
        offset (int, optional): 忽略前面的 `offset` 个帖子，默认 0

    返回:
        Dict[str, Any]: 搜索结果
    '''
    ...

# 搜索帖子
def get_list(**kwargs: Any) -> Dict[str, Any]:
    # 去除 None 值字段
    kwargs = {key: value for key, value in kwargs.items() if value is not None}
    # 将下划线字段名转换为小驼峰字段名
    kwargs = {
        (
            "".join(x.capitalize() if i > 0 else x for i, x in enumerate(key.split("_")))
        ): value for key, value in kwargs.items() if value is not None
    }
    response = Api(API['post']['list']).post(data=kwargs)
    return response.json()

# 异步搜索社区谱面
@overload
async def get_list_async(
    *,
    search: str='',
    category_name: Literal['SELF_POST']='SELF_POST',
    category_id: Literal['chart']='chart',
    tags: List[Tag]=[],
    order: Literal['TIME_DESC', 'TIME_ASC']='TIME_DESC',
    limit: int=20,
    offset: int=0
) -> Dict[str, Any]:
    '''搜索社区谱面
        ```python
        # 以 'Arghena' 为关键词，搜索社区谱面
        Post.search(search='Arghena', caregory_name='SELF_POST', category_id='chart')
        ```

    参数:
        search (str, optional): 搜索关键词，默认为空
        category_name (Literal[&#39;SELF_POST&#39;], optional): 搜索的帖子类型 `SELF_POST`
        category_id (Literal[&#39;chart&#39;, &#39;text&#39;], optional): 搜索的画廊种类 `chart`
        tags (List[Tag], optional): 搜索的标签，默认为空
        order (Literal[&#39;TIME_DESC&#39;, &#39;TIME_ASC&#39;], optional): 帖子排序，默认时间倒序
        limit (int, optional): 展示出的帖子数，默认 20
        offset (int, optional): 忽略前面的 `offset` 个帖子，默认 0

    返回:
        Dict[str, Any]: 搜索结果
            ```python
            result: bool # 是否有响应
            count: int # 搜索到的谱面总数
            posts: List[Dict[str, Any]] # 列举出的谱面
            ```
    '''
    ...

# 异步搜索用户帖子
@overload
async def get_list_async(
    *,
    username: str,
    limit: int=20,
    offset: int=0,
    order: Literal['TIME_DESC', 'TIME_ASC']='TIME_DESC'
) -> Dict[str, Any]:
    '''搜索用户帖子

    参数:        
        username (str): 用户名
        limit (int, optional): 展示出的帖子数，默认 20
        offset (int, optional): 忽略前面的 `offset` 个帖子，默认 0
        order (Literal[&#39;TIME_DESC&#39;, &#39;TIME_ASC&#39;], optional): 帖子排序，默认时间倒序

    返回:
        Dict[str, Any]: 搜索结果
            ```python
            result: bool # 是否有响应
            count: int # 搜索到的帖子总数
            posts: List[Dict[str, Any]] # 列举出的帖子
            ```
    '''
    ...

# 异步搜索帖子
@overload
async def get_list_async(
    *,
    search: Optional[str]=None,
    following: Optional[bool]=None,
    category_name: Optional[str]=None,
    category_id: Optional[str]=None,
    tags: Optional[List[Tag]]=None,
    username: Optional[str]=None,
    order: Literal['TIME_DESC', 'TIME_ASC'],
    limit: int=20,
    offset: int=0
) -> Dict[str, Any]:
    '''搜索帖子

    参数:        
        order (Literal[&#39;TIME_DESC&#39;, &#39;TIME_ASC&#39;]): 帖子排序
        search (Optional[str], optional): 搜索关键词
        following (Optional[bool], optional): 是否关注
        category_name (Optional[str], optional): 画廊名称
        category_id (Optional[str], optional): 画廊 ID
        tags (Optional[List[Tag]], optional): 帖子标签
        username (Optional[str], optional): 用户名
        limit (int, optional): 展示出的帖子数，默认 20
        offset (int, optional): 忽略前面的 `offset` 个帖子，默认 0

    返回:
        Dict[str, Any]: 搜索结果
    '''
    ...

# 异步搜索帖子
async def get_list_async(**kwargs: Any) -> Dict[str, Any]:
    # 去除 None 值字段
    kwargs = {key: value for key, value in kwargs.items() if value is not None}
    # 将下划线字段名转换为小驼峰字段名
    kwargs = {
        (
            "".join(x.capitalize() if i > 0 else x for i, x in enumerate(key.split("_")))
        ): value for key, value in kwargs.items() if value is not None
    }
    response = await Api(API['post']['list']).apost(data=kwargs)
    if isinstance(response, Response):
        return response.json()
    else:
        return await response.json()

# 搜索标签
def search_tags(
    type: str,
    data: str='',
    fuzzy: bool=True
) -> List[Tag]:
    '''搜索已有标签

    参数:
        type (str): 标签类型
        data (str, optional): 搜索标签数据关键词
        fuzzy (bool, optional): 是否使用模糊搜索

    返回:
        List[Tag]: 标签类 `Tag` 列表
    '''
    response = Api(API['post']['tag']).get(
        params={
            'type': type,
            'data': data,
            'fuzzy': fuzzy
        }
    )
    if (tags := response.json().get('tags', None)) is not None:
        return [Tag(tag) for tag in tags]
    else:
        raise RequestException(API['post']['tag'], '搜索标签时出现未知错误')

# 异步搜索标签
async def search_tags_async(
    type: str,
    data: str='',
    fuzzy: bool=True
) -> List[Tag]:
    '''搜索已有标签

    参数:
        type (str): 标签类型
        data (str, optional): 搜索标签数据关键词
        fuzzy (bool, optional): 是否使用模糊搜索

    返回:
        List[Tag]: 标签类 `Tag` 列表
    '''
    response = await Api(API['post']['tag']).aget(
        params={
            'type': type,
            'data': data,
            'fuzzy': fuzzy
        }
    )
    
    if isinstance(response, Response):
        _data = response.json()
    else:
        _data = await response.json()
    
    if (tags := _data.get('tags', None)) is not None:
        return [Tag(tag) for tag in tags]
    else:
        raise RequestException(API['post']['tag'], '搜索标签时出现未知错误')

# 发表谱面
@overload
def post(
    me: 'Me',
    *,
    artists: str,
    category_id: Literal['chart']='chart',
    category_name: Literal['SELF_POST']='SELF_POST',
    chart: Chart,
    content: List[Content],
    diff: Literal[0, 1, 2, 3, 4],
    level: int,
    song: Union[CustomSong, ProvidedSong],
    tags: List[Tag]=[],
    title: str
) -> int:
    '''发表谱面

    参数:
        me (Me): 自身用户对象
        artists (str): 歌手
        category_id (Literal[&#39;chart&#39;], optional): 谱面画廊 ID `chart`
        category_name (Literal[&#39;SELF_POST&#39;], optional): 谱面画廊名称 `SELF_POST`
        chart (Chart): 谱面
        content (List[Content]): 帖子内容
        diff (Literal[0, 1, 2, 3, 4]): 难度
        level (int): 等级
        song (Union[CustomSong, ProvidedSong]): 歌曲
        tags (List[Tag], optional): 谱面标签
        title (str): 谱面标题

    返回:
        int: 谱面 ID
    '''
    ...

# 发表文本帖子
@overload
def post(
    me: 'Me',
    *,
    category_id: Literal['text']='text',
    category_name: Literal['SELF_POST']='SELF_POST',
    content: List[Content],
    tags: List[Tag]=[],
    title: str
) -> int:
    '''发表文本帖子

    参数:
        me (Me): 自身用户对象
        category_id (Literal[&#39;text&#39;], optional): 帖子画廊 ID `text`
        category_name (Literal[&#39;SELF_POST&#39;], optional): 帖子画廊名称 `SELF_POST`
        content (List[Content]): 帖子内容
        tags (List[Tag], optional): 帖子标签
        title (str): 帖子标题

    返回:
        int: 帖子 ID
    '''
    ...

# 发表帖子
@overload
def post(
    me: 'Me',
    *,
    artists: Optional[str]=None,
    category_id: str,
    category_name: str,
    chart: Optional[Chart]=None,
    content: List[Content],
    diff: Optional[Literal[0, 1, 2, 3, 4]]=None,
    level: Optional[int]=None,
    song: Optional[Union[CustomSong, ProvidedSong]]=None,
    tags: Optional[List[Tag]]=None,
    title: Optional[str]=None
) -> int:
    '''发表帖子

    参数:
        me (Me): 自身用户对象
        artists (Optional[str], optional): 歌手
        category_id (str): 帖子画廊 ID
        category_name (str): 帖子画廊名称
        chart (Optional[Chart], optional): 谱面
        content (List[Content]): 帖子内容
        diff (Optional[Literal[0, 1, 2, 3, 4]], optional): 难度
        level (Optional[int], optional): 等级
        song (Optional[Union[CustomSong, ProvidedSong]], optional): 歌曲
        tags (Optional[List[Tag]], optional): 帖子标签
        title (Optional[str], optional): 帖子标题

    返回:
        int: 帖子 ID
    '''
    ...

# 发表帖子
def post(
    me: 'Me',
    **kwargs: Any
) -> int:
    # 转换特定字段
    if 'chart' in kwargs:
        kwargs['chart'] = kwargs['chart'].to_list()
    if 'content' in kwargs:
        content = kwargs['content']
        kwargs['content'] = [seg.__dict__ for seg in content]
    
    # 去除 None 值字段
    kwargs = {key: value for key, value in kwargs.items() if value is not None}
    # 将下划线字段名转换为小驼峰字段名
    kwargs = {
        (
            "".join(x.capitalize() if i > 0 else x for i, x in enumerate(key.split("_")))
        ): value for key, value in kwargs.items() if value is not None
    }
    response = Api(API['post']['post']).post(
        cookies=me.cookies,
        data=kwargs
    )
    if (id := response.json().get('id', None)) is None:
        raise ValueError('发表帖子时出现未知错误。')
    return id

# 异步发表谱面
@overload
async def apost(
    me: 'Me',
    *,
    artists: str,
    category_id: Literal['chart']='chart',
    category_name: Literal['SELF_POST']='SELF_POST',
    chart: Chart,
    content: List[Content],
    diff: Literal[0, 1, 2, 3, 4],
    level: int,
    song: Union[CustomSong, ProvidedSong],
    tags: List[Tag]=[],
    title: str
) -> int:
    '''发表谱面

    参数:
        me (Me): 自身用户对象
        artists (str): 歌手
        category_id (Literal[&#39;chart&#39;], optional): 谱面画廊 ID `chart`
        category_name (Literal[&#39;SELF_POST&#39;], optional): 谱面画廊名称 `SELF_POST`
        chart (Chart): 谱面
        content (List[Content]): 帖子内容
        diff (Literal[0, 1, 2, 3, 4]): 难度
        level (int): 等级
        song (Union[CustomSong, ProvidedSong]): 歌曲
        tags (List[Tag], optional): 谱面标签
        title (str): 谱面标题

    返回:
        int: 谱面 ID
    '''
    ...

# 异步发表文本帖子
@overload
async def apost(
    me: 'Me',
    *,
    category_id: Literal['text']='text',
    category_name: Literal['SELF_POST']='SELF_POST',
    content: List[Content],
    tags: List[Tag]=[],
    title: str
) -> int:
    '''发表文本帖子

    参数:
        me (Me): 自身用户对象
        category_id (Literal[&#39;text&#39;], optional): 帖子画廊 ID `text`
        category_name (Literal[&#39;SELF_POST&#39;], optional): 帖子画廊名称 `SELF_POST`
        content (List[Content]): 帖子内容
        tags (List[Tag], optional): 帖子标签
        title (str): 帖子标题

    返回:
        int: 帖子 ID
    '''
    ...

# 异步发表帖子
@overload
async def apost(
    me: 'Me',
    *,
    artists: Optional[str]=None,
    category_id: str,
    category_name: str,
    chart: Optional[Chart]=None,
    content: List[Content],
    diff: Optional[Literal[0, 1, 2, 3, 4]]=None,
    level: Optional[int]=None,
    song: Optional[Union[CustomSong, ProvidedSong]]=None,
    tags: Optional[List[Tag]]=None,
    title: Optional[str]=None
) -> int:
    '''发表帖子

    参数:
        me (Me): 自身用户对象
        artists (Optional[str], optional): 歌手
        category_id (str): 帖子画廊 ID
        category_name (str): 帖子画廊名称
        chart (Optional[Chart], optional): 谱面
        content (List[Content]): 帖子内容
        diff (Optional[Literal[0, 1, 2, 3, 4]], optional): 难度
        level (Optional[int], optional): 等级
        song (Optional[Union[CustomSong, ProvidedSong]], optional): 歌曲
        tags (Optional[List[Tag]], optional): 帖子标签
        title (Optional[str], optional): 帖子标题

    返回:
        int: 帖子 ID
    '''
    ...

# 异步发表帖子
async def apost(
    me: 'Me',
    **kwargs: Any
) -> int:
    # 转换特定字段
    if 'chart' in kwargs:
        kwargs['chart'] = kwargs['chart'].to_list()
    if 'content' in kwargs:
        content = kwargs['content']
        kwargs['content'] = [seg.__dict__ for seg in content]
    
    # 去除 None 值字段
    kwargs = {key: value for key, value in kwargs.items() if value is not None}
    # 将下划线字段名转换为小驼峰字段名
    kwargs = {
        (
            "".join(x.capitalize() if i > 0 else x for i, x in enumerate(key.split("_")))
        ): value for key, value in kwargs.items() if value is not None
    }
    response = await Api(API['post']['post']).apost(
        cookies=me.cookies,
        data=kwargs
    )
    
    if isinstance(response, Response):
        _response = response.json()
    else:
        _response = await response.json()
    
    if (id := _response.get('id', None)) is None:
        raise ValueError('发表帖子时出现未知错误。')
    return id

# 查询帖子顺序
def find_post(category_name: str, category_id: str, id: int) -> int:
    '''查询帖子顺序

    参数:
        category_name (str): 画廊名称
        category_id (str): 画廊 ID
        id (int): 查询的帖子 ID

    返回:
        int: 帖子在该画廊的时间顺序
    '''
    params = {
        'categoryName': category_name,
        'categoryId': category_id,
        'id': id
    }
    response = Api(API['post']['find']).get(params=params)
    if (position := response.json().get('position', None)) is None:
        raise ValueError('查询帖子顺序时出现未知错误。')
    return position

# 异步查询帖子顺序
async def find_post_async(category_name: str, category_id: str, id: int) -> int:
    '''查询帖子顺序

    参数:
        category_name (str): 画廊名称
        category_id (str): 画廊 ID
        id (int): 查询的帖子 ID

    返回:
        int: 帖子在该画廊的时间顺序
    '''
    params = {
        'categoryName': category_name,
        'categoryId': category_id,
        'id': id
    }
    response = await Api(API['post']['find']).aget(params=params)
    
    if isinstance(response, Response):
        _response = response.json()
    else:
        _response = await response.json()
    
    if (position := _response.get('position', None)) is None:
        raise ValueError('查询帖子顺序时出现未知错误。')
    return position

# 社区帖子类
class Post:
    '''社区帖子类

    参数:
        id (str): 社区帖子 ID
    '''
    # 初始化
    def __init__(self, id: int) -> None:
        '''社区帖子类

        参数:
            id (int): 社区帖子 ID
        '''
        self.id: int = id
        '''社区帖子 ID'''
        self.__post: Dict[str, Any] = {}
        '''社区帖子详细内容'''
        return
    
    # 谱面对象
    @property
    def chart(self) -> Chart:
        '''谱面对象'''
        post = self.__post
        if (chart := post.get('chart', None)) is not None:
            return Chart.normalize(chart)
        else:
            raise PostHasNoChartError(post)
    
    # 帖子标签
    @property
    def tags(self) -> List[Tag]:
        '''帖子标签'''
        if (tags := self.__post.get('tags', None)) is not None:
            return [Tag(tag) for tag in tags]
        else:
            return []
    
    # 帖子内容
    @property
    def content(self) -> str:
        '''帖子内容'''
        result: str = ''
        if (content := list(self.__post.get('content', None))) is not None:
            for seg in content:
                if seg.get('type', None) in ['text', 'link']:
                    result += seg.get('data', '') + '\n'
                elif seg.get('type', None) == 'emoji':
                    result += f':{seg.get("data", "")}:'
                elif seg.get('type', None) == 'br':
                    result += '\n'
        return result
    
    # 获取帖子基础信息
    def get_basic(self) -> Dict[str, Any]:
        '''获取帖子基础信息

        返回:
            Dict[str, Any]: 基础信息
        '''
        response = Api(API['post']['basic']).get(params={'id': self.id,})
        return response.json()
    
    # 异步获取帖子基础信息
    async def get_basic_async(self) -> Dict[str, Any]:
        '''获取帖子基础信息

        返回:
            Dict[str, Any]: 基础信息
        '''
        response = await Api(API['post']['basic']).aget(params={'id': self.id,})
        if isinstance(response, Response):
            return response.json()
        else:
            return await response.json()
    
    # 获取帖子信息
    def get_details(self) -> Dict[str, Any]:
        '''获取帖子信息

        返回:
            Dict[str, Any]: 帖子详细信息
        '''
        response = Api(API['post']['details']).get(params={'id': self.id,})
        if (post := response.json().get('post', None)) is not None:
            self.__post = dict(post)
        else:
            raise NoDataException('帖子信息')
        return self.__post
    
    # 异步获取帖子信息
    async def get_details_async(self) -> Dict[str, Any]:
        '''获取帖子信息

        返回:
            Dict[str, Any]: 帖子详细信息
        '''
        response = await Api(API['post']['details']).aget(params={'id': self.id,})
        if isinstance(response, Response):
            _data = response.json()
        else:
            _data = await response.json()
        
        if (post := _data.get('post', None)) is not None:
            self.__post = dict(post)
        else:
            raise NoDataException('帖子信息')
        return self.__post
    
    # 获取缓存信息
    def __get_post_cache(self) -> Dict[str, Any]:
        '''获取缓存信息

        返回:
            Dict[str, Any]: 缓存信息
        '''
        if not self.__post:
            self.get_details()
        return self.__post
    
    # 异步获取缓存信息
    async def __get_post_cache_async(self) -> Dict[str, Any]:
        '''获取缓存信息

        返回:
            Dict[str, Any]: 缓存信息
        '''
        if not self.__post:
            await self.get_details_async()
        return self.__post
    
    # 获取歌曲信息对象
    def get_song(self) -> SongRes:
        '''获取歌曲信息对象

        返回:
            SongInfo: 歌曲音频与封面字节
        '''
        post = self.__get_post_cache()
        if (song := post.get('song', None)) is None:
            raise PostHasNoSongError(post)
        
        if (type := song.get('type', None)) is None:
            raise TypeError('该帖子没有歌曲类型。')
        
        result: Dict[str, Union[bytes, None]] = {}
        if type == 'custom': # 自定义歌曲
            # 获取歌曲音频
            if (audio := song.get('audio', None)) is None:
                result['audio'] = None
            else:
                response = Api(audio).get()
                result['audio'] = response.content
            # 获取歌曲封面
            if (cover := song.get('cover', None)) is None:
                result['cover'] = None
            else:
                response = Api(cover).get()
                result['cover'] = response.content
        elif type == 'bandori': # BanG Dream! 歌曲
            # 获取歌曲 ID
            if (id := song.get('id', None)) is None:
                raise ValueError('未能获取歌曲 ID。')
            # 获取歌曲信息
            info = Api(API['songs']['info'].format(id=id)).get().json()
            # 获取歌曲所在服务器
            if (published_at := info.get('publishedAt', None)) is None:
                raise NoDataException('歌曲发布时间')
            # 根据 publishedAt 数据判断服务器
            if published_at[0] is not None: server = 'jp'
            elif published_at[1] is not None: server = 'en'
            elif published_at[2] is not None: server = 'tw'
            elif published_at[3] is not None: server = 'cn'
            elif published_at[4] is not None: server = 'kr'
            else:
                raise NoDataException('歌曲服务器')
            # 获取歌曲音频
            result['audio'] = Assets(
                ASSETS['songs']['sound'].format(id=id), server
            ).get()
            # 获取歌曲封面
            # 获取数据包序列号
            quotient, remainder = divmod(id, 10)
            if remainder == 0:
                index = id
            else:
                index = (quotient + 1) * 10
            
            if (jacket_image := info.get('jacketImage', None)) is None:
                raise NoDataException('歌曲封面资源')
            result['cover'] = Assets(
                ASSETS['songs']['musicjacket'].format(
                    index=index, jacket_image=jacket_image[-1]
                ), server
            ).get()
        elif type == 'llsif': # LoveLive! 歌曲
            # 获取歌曲 ID
            if (id := song.get('id', None)) is None:
                raise ValueError('未能获取歌曲 ID。')
            # 获取歌曲信息
            info = Api(API['misc']['llsif'].format(index=10)).get().json()[str(id)]
            # 获取歌曲资源库
            live_icon_asset = info.get('live_icon_asset', None)
            sound_asset = info.get('sound_asset', None)
            # 获取歌曲音频
            result['audio'] = Assets(sound_asset, 'llsif').get()
            # 获取歌曲封面
            result['cover'] = Assets(live_icon_asset, 'llsif').get()
        else:
            raise AssetsNotExistError(f'{type} 歌曲')
        
        return SongRes(**result)
    
    # 异步获取歌曲信息对象
    async def get_song_async(self) -> SongRes:
        '''获取歌曲信息对象

        返回:
            SongInfo: 歌曲音频与封面字节
        '''
        post = await self.__get_post_cache_async()
        if (song := post.get('song', None)) is None:
            raise PostHasNoSongError(post)
        
        if (type := song.get('type', None)) is None:
            raise TypeError('该帖子没有歌曲类型。')
        
        result: Dict[str, Union[bytes, None]] = {}
        if type == 'custom': # 自定义歌曲
            # 获取歌曲音频
            if (audio := song.get('audio', None)) is None:
                result['audio'] = None
            else:
                response = await Api(audio).aget()
                if isinstance(response, Response):
                    result['audio'] = response.content
                else:
                    result['audio'] = await response.read()
            # 获取歌曲封面
            if (cover := song.get('cover', None)) is None:
                result['cover'] = None
            else:
                response = await Api(cover).aget()
                if isinstance(response, Response):
                    result['cover'] = response.content
                else:
                    result['cover'] = await response.read()
        elif type == 'bandori': # BanG Dream! 歌曲
            # 获取歌曲 ID
            if (id := song.get('id', None)) is None:
                raise ValueError('未能获取歌曲 ID。')
            # 获取歌曲信息
            info = await Api(API['songs']['info'].format(id=id)).aget()
            if isinstance(info, Response):
                info = info.json()
            else:
                info = await info.json()
            # 获取歌曲所在服务器
            if (published_at := info.get('publishedAt', None)) is None:
                raise NoDataException('歌曲发布时间')
            # 根据 publishedAt 数据判断服务器
            if published_at[0] is not None: server = 'jp'
            elif published_at[1] is not None: server = 'en'
            elif published_at[2] is not None: server = 'tw'
            elif published_at[3] is not None: server = 'cn'
            elif published_at[4] is not None: server = 'kr'
            else:
                raise NoDataException('歌曲服务器')
            # 获取歌曲音频
            result['audio'] = await Assets(
                ASSETS['songs']['sound'].format(id=id), server
            ).aget()
            # 获取歌曲封面
            # 获取数据包序列号
            quotient, remainder = divmod(id, 10)
            if remainder == 0:
                index = id
            else:
                index = (quotient + 1) * 10
            
            if (jacket_image := info.get('jacketImage', None)) is None:
                raise NoDataException('歌曲封面资源')
            result['cover'] = await Assets(
                ASSETS['songs']['musicjacket'].format(
                    index=index, jacket_image=jacket_image[-1]
                ), server
            ).aget()
        elif type == 'llsif': # LoveLive! 歌曲
            # 获取歌曲 ID
            if (id := song.get('id', None)) is None:
                raise ValueError('未能获取歌曲 ID。')
            # 获取歌曲信息
            info = await Api(API['misc']['llsif'].format(index=10)).aget()
            if isinstance(info, Response):
                info = info.json()
            else:
                info = await info.json()
            info = info[str(id)]
            # 获取歌曲资源库
            live_icon_asset = info.get('live_icon_asset', None)
            sound_asset = info.get('sound_asset', None)
            # 获取歌曲音频
            result['audio'] = await Assets(sound_asset, 'llsif').aget()
            # 获取歌曲封面
            result['cover'] = await Assets(live_icon_asset, 'llsif').aget()
        else:
            raise AssetsNotExistError(f'{type} 歌曲')
        
        return SongRes(**result)
    
    # 获取帖子评论
    def get_comment(
        self,
        limit: int=20,
        offset: int=0,
        order: Literal['TIME_DESC', 'TIME_ASC']='TIME_ASC'
    ) -> Dict[str, Any]:
        '''获取帖子评论

        参数:
            limit (int, optional): 展示出的评论数，默认为 20
            offset (int, optional): 忽略前面的 `offset` 条评论，默认为 0
            order (Literal[&#39;TIME_DESC&#39;, &#39;TIME_ASC&#39;], optional): 排序顺序，默认时间顺序

        返回:
            Dict[str, Any]: 搜索结果
                ```python
                {
                    "result": ... # bool 是否有响应
                    "count": ... # int 搜索到的评论总数
                    "posts": ... # List[Dict[str, Any]] 列举出的评论
                }
                ```
        '''
        return get_list(
            category_name='POST_COMMENT',
            category_id=str(self.id),
            order=order,
            limit=limit,
            offset=offset
        )
    
    # 异步获取帖子评论
    async def get_comment_async(
        self,
        limit: int=20,
        offset: int=0,
        order: Literal['TIME_DESC', 'TIME_ASC']='TIME_ASC'
    ) -> Dict[str, Any]:
        '''获取帖子评论

        参数:
            limit (int, optional): 展示出的评论数，默认为 20
            offset (int, optional): 忽略前面的 `offset` 条评论，默认为 0
            order (Literal[&#39;TIME_DESC&#39;, &#39;TIME_ASC&#39;], optional): 排序顺序，默认时间顺序

        返回:
            Dict[str, Any]: 搜索结果
                ```python
                {
                    "result": ... # bool 是否有响应
                    "count": ... # int 搜索到的评论总数
                    "posts": ... # List[Dict[str, Any]] 列举出的评论
                }
                ```
        '''
        return await get_list_async(
            category_name='POST_COMMENT',
            category_id=str(self.id),
            order=order,
            limit=limit,
            offset=offset
        )
    
    # 评论帖子
    def comment(self, me: 'Me', content: List[Content]) -> int:
        '''评论帖子

        参数:
            me (Me): 自身用户对象
            content (List[Content]): 评论内容

        返回:
            int: 评论 ID
        '''
        return post(
            me,
            category_id=str(self.id),
            category_name='POST_COMMENT',
            content=content
        )
    
    # 异步评论帖子
    async def acomment(self, me: 'Me', content: List[Content]) -> int:
        '''评论帖子

        参数:
            me (Me): 自身用户对象
            content (List[Content]): 评论内容

        返回:
            int: 评论 ID
        '''
        return await apost(
            me,
            category_id=str(self.id),
            category_name='POST_COMMENT',
            content=content
        )
    
    # 喜欢 / 取消喜欢帖子
    def like(self, me: 'Me', value: bool=True) -> None:
        '''喜欢 / 取消喜欢帖子

        参数:
            me (Me): 自身用户对象
            value (bool, optional): 值 `True`: 喜欢帖子 `False`: 取消喜欢帖子
        '''
        Api(API['post']['like']).post(
            data={'id': self.id, 'value': value},
            cookies=me.cookies
        )
    
    # 异步喜欢 / 取消喜欢帖子
    async def alike(self, me: 'Me', value: bool=True) -> None:
        '''喜欢 / 取消喜欢帖子

        参数:
            me (Me): 自身用户对象
            value (bool, optional): 值 `True`: 喜欢帖子 `False`: 取消喜欢帖子
        '''
        await Api(API['post']['like']).apost(
            data={'id': self.id, 'value': value},
            cookies=me.cookies
        )



'''`bestdori.ayachan.settings` 设置项模块'''

from typing import Dict, Union, Optional

from bestdori.settings import AsyncClient

proxy: Optional[Union[Dict[str, str], str]] = None
'''代理服务器'''
async_client: AsyncClient = AsyncClient.HTTPX
'''异步客户端

默认为 `httpx`，可选 `aiohttp`
'''
timeout: int = 10
'''超时时间'''



'''`bestdori.festival` 团队佳节活动专属数据获取 API

包括舞台数据、活动歌曲数据'''

from typing import Dict, List, TypedDict

from httpx import Response

from .utils.utils import API
from .utils.network import Api

# 歌曲循环数据类型
class RotationMusic(TypedDict):
    musicId: int
    startAt: str
    endAt: str

# 舞台数据类型
class Stage(TypedDict):
    type: str
    startAt: str
    endAt: str

# 获取歌曲循环数据
def get_rotation_musics(id: int) -> List[RotationMusic]:
    '''获取歌曲循环数据

    参数:
        id (int): 活动 ID

    返回:
        List[Dict[str, Any]]: 歌曲循环数据
    '''
    return Api(API['festival']['rotation_musics'].format(id=id)).get().json()

# 异步获取歌曲循环数据
async def get_rotation_musics_async(id: int) -> List[RotationMusic]:
    '''获取歌曲循环数据

    参数:
        id (int): 活动 ID

    返回:
        List[Dict[str, Any]]: 歌曲循环数据
    '''
    response = await Api(API['festival']['rotation_musics'].format(id=id)).aget()
    if isinstance(response, Response):
        return response.json()
    return await response.json()

# 获取舞台数据
def get_stages(id: int) -> List[Stage]:
    '''获取舞台数据

    参数:
        id (int): 活动 ID

    返回:
        List[Dict[str, Any]]: 舞台数据
    '''
    return Api(API['festival']['stages'].format(id=id)).get().json()

# 异步获取舞台数据
async def get_stages_async(id: int) -> List[Stage]:
    '''获取舞台数据

    参数:
        id (int): 活动 ID

    返回:
        List[Dict[str, Any]]: 舞台数据
    '''
    response = await Api(API['festival']['stages'].format(id=id)).aget()
    if isinstance(response, Response):
        return response.json()
    return await response.json()



'''`bestdori.ayachan.exceptions`

ayachan API 错误信息相关操作'''

# Sonolus 相关错误
class SonolusException(Exception):
    '''Sonolus 相关错误'''
    # 初始化
    def __init__(self, msg: str) -> None:
        super().__init__(msg)



'''`bestdori.characters`

BanG Dream! 角色相关操作'''
from typing import Any, Dict, Tuple, Literal

from aiohttp import ClientResponseError
from httpx import Response, HTTPStatusError

from .utils import hexto_rgb
from .utils.utils import API, RES, ASSETS
from .post import get_list, get_list_async
from .utils.network import Api, Res, Assets
from .exceptions import (
    NoDataException,
    CharacterNotExistError
)

# 获取总角色信息
def get_all(index: Literal[2]=2) -> Dict[str, Dict[str, Any]]:
    '''获取总角色信息

    参数:
        index (Literal[2], optional): 指定获取哪种 `all.json`
            `2`: 获取所有已有角色信息 `all.2.json`

    返回:
        Dict[str, Dict[str, Any]]: 获取到的总角色信息
    '''
    return Api(API['characters']['all'].format(index=index)).get().json()

# 异步获取总角色信息
async def get_all_async(index: Literal[2]=2) -> Dict[str, Dict[str, Any]]:
    '''获取总角色信息

    参数:
        index (Literal[2], optional): 指定获取哪种 `all.json`
            `2`: 获取所有已有角色信息 `all.2.json`

    返回:
        Dict[str, Dict[str, Any]]: 获取到的总角色信息
    '''
    response = await Api(API['characters']['all'].format(index=index)).aget()
    if isinstance(response, Response):
        return response.json()
    else:
        return await response.json()
    

# 获取主要角色信息
def get_main(index: Literal[3]=3) -> Dict[str, Dict[str, Any]]:
    '''获取主要角色信息

    参数:
        index (Literal[3], optional): 指定获取哪种 `all.json`
            `3`: 获取所有已有主要角色信息 `all.3.json`

    返回:
        Dict[str, Dict[str, Any]]: 获取到的主要角色信息
    '''
    return Api(API['characters']['main'].format(index=index)).get().json()

# 异步获取主要角色信息
async def get_main_async(index: Literal[3]=3) -> Dict[str, Dict[str, Any]]:
    '''获取主要角色信息

    参数:
        index (Literal[3], optional): 指定获取哪种 `all.json`
            `3`: 获取所有已有主要角色信息 `all.3.json`

    返回:
        Dict[str, Dict[str, Any]]: 获取到的主要角色信息
    '''
    response = await Api(API['characters']['main'].format(index=index)).aget()
    if isinstance(response, Response):
        return response.json()
    else:
        return await response.json()

# 角色类
class Character:
    '''角色类

    参数:
        id (int): 角色 ID
    '''
    # 初始化
    def __init__(self, id: int) -> None:
        '''角色类

        参数:
            id (int): 角色 ID
        '''
        self.id: int = id
        '''角色 ID'''
        self.__info: Dict[str, Any] = {}
        '''角色信息'''
        return
    
    # 角色名称
    @property
    def name(self) -> str:
        '''角色名称'''
        info = self.__info
        # 获取 characterName 数据
        if (character_name := info.get('characterName', None)) is None:
            raise NoDataException('角色名称')
        # 获取第一个非 None 角色名称
        try:
            return next(filter(lambda x: x is not None, character_name))
        except StopIteration:
            raise NoDataException('角色名称')
    
    # 角色代表色
    @property
    def color(self) -> Tuple[int, int, int]:
        '''角色代表色'''
        info = self.__info
        # 获取 colorCode 数据
        if (color_code := info.get('colorCode', None)) is None:
            raise NoDataException('角色颜色')
        # 将 colorCode 转换为颜色元组
        try:
            return hexto_rgb(color_code)
        except ValueError:
            raise NoDataException('角色颜色')
    
    # 获取角色信息
    def get_info(self) -> Dict[str, Any]:
        '''获取角色信息

        返回:
            Dict[str, Any]: 角色详细信息
        '''
        try:
            response = Api(
                API['characters']['info'].format(id=self.id)
            ).get()
        except HTTPStatusError as exception:
            if exception.response.status_code == 404:
                raise CharacterNotExistError(self.id)
            else:
                raise exception
        
        self.__info = dict(response.json())
        return self.__info
    
    # 异步获取角色信息
    async def get_info_async(self) -> Dict[str, Any]:
        '''获取角色信息

        返回:
            Dict[str, Any]: 角色详细信息
        '''
        try:
            response = await Api(
                API['characters']['info'].format(id=self.id)
            ).aget()
        except HTTPStatusError as exception:
            if exception.response.status_code == 404:
                raise CharacterNotExistError(self.id)
            else:
                raise exception
        except ClientResponseError as exception:
            if exception.status == 404:
                raise CharacterNotExistError(self.id)
            else:
                raise exception
        
        if isinstance(response, Response):
            self.__info = dict(response.json())
        else:
            self.__info = dict(await response.json())
        return self.__info
    
    # 获取角色评论
    def get_comment(
        self,
        limit: int=20,
        offset: int=0,
        order: Literal['TIME_DESC', 'TIME_ASC']='TIME_ASC'
    ) -> Dict[str, Any]:
        '''获取角色评论

        参数:
            limit (int, optional): 展示出的评论数，默认为 20
            offset (int, optional): 忽略前面的 `offset` 条评论，默认为 0
            order (Literal[&#39;TIME_DESC&#39;, &#39;TIME_ASC&#39;], optional): 排序顺序，默认时间顺序

        返回:
            Dict[str, Any]: 搜索结果
                ```python
                {
                    "result": ... # bool 是否有响应
                    "count": ... # int 搜索到的评论总数
                    "posts": ... # List[Dict[str, Any]] 列举出的评论
                }
                ```
        '''
        return get_list(
            category_name='CHARACTER_COMMENT',
            category_id=str(self.id),
            order=order,
            limit=limit,
            offset=offset
        )
    
    # 异步获取角色评论
    async def get_comment_async(
        self,
        limit: int=20,
        offset: int=0,
        order: Literal['TIME_DESC', 'TIME_ASC']='TIME_ASC'
    ) -> Dict[str, Any]:
        '''获取角色评论

        参数:
            limit (int, optional): 展示出的评论数，默认为 20
            offset (int, optional): 忽略前面的 `offset` 条评论，默认为 0
            order (Literal[&#39;TIME_DESC&#39;, &#39;TIME_ASC&#39;], optional): 排序顺序，默认时间顺序

        返回:
            Dict[str, Any]: 搜索结果
                ```python
                {
                    "result": ... # bool 是否有响应
                    "count": ... # int 搜索到的评论总数
                    "posts": ... # List[Dict[str, Any]] 列举出的评论
                }
                ```
        '''
        return await get_list_async(
            category_name='CHARACTER_COMMENT',
            category_id=str(self.id),
            order=order,
            limit=limit,
            offset=offset
        )
    
    # 获取角色图标
    def get_icon(self) -> bytes:
        '''获取角色图标

        返回:
            bytes: 角色图标字节数据 `bytes`
        '''
        return Res(RES['icon']['png'].format(name=f'chara_icon_{self.id}')).get()
    
    # 异步获取角色图标
    async def get_icon_async(self) -> bytes:
        '''获取角色图标

        返回:
            bytes: 角色图标字节数据 `bytes`
        '''
        return await Res(RES['icon']['png'].format(name=f'chara_icon_{self.id}')).aget()
    
    # 获取角色主视觉图
    def get_kv_image(self) -> bytes:
        '''获取角色主视觉图

        返回:
            bytes: 主视觉图资源字节 `bytes`
        '''
        return Assets(
            ASSETS['characters']['character_kv_image'].format(id=self.id), 'jp'
        ).get()
    
    # 异步获取角色主视觉图
    async def get_kv_image_async(self) -> bytes:
        '''获取角色主视觉图

        返回:
            bytes: 主视觉图资源字节 `bytes`
        '''
        return await Assets(
            ASSETS['characters']['character_kv_image'].format(id=self.id), 'jp'
        ).aget()



'''`bestdori.comics`

BanG Dream! 漫画相关操作'''
from typing import Any, Dict, Literal

from httpx import Response

from .utils.utils import API, ASSETS
from .utils.network import Api, Assets
from .post import get_list, get_list_async
from .exceptions import (
    NoDataException,
    ComicNotExistError,
    ServerNotAvailableError
)

# 获取总漫画信息
def get_all(index: Literal[5]=5) -> Dict[str, Dict[str, Any]]:
    '''获取总漫画信息

    参数:
        index (Literal[5], optional): 指定获取哪种 `all.json`
            `5`: 获取所有已有漫画信息 `all.5.json`

    返回:
        Dict[str, Dict[str, Any]]: 获取到的总漫画信息
    '''
    return Api(API['all']['comics'].format(index=index)).get().json()

# 异步获取总漫画信息
async def get_all_async(index: Literal[5]=5) -> Dict[str, Dict[str, Any]]:
    '''获取总漫画信息

    参数:
        index (Literal[5], optional): 指定获取哪种 `all.json`
            `5`: 获取所有已有漫画信息 `all.5.json`

    返回:
        Dict[str, Dict[str, Any]]: 获取到的总漫画信息
    '''
    response = await Api(API['all']['comics'].format(index=index)).aget()
    if isinstance(response, Response):
        return response.json()
    else:
        return await response.json()

# 漫画类
class Comic:
    '''漫画类

    参数:
        id (int): 漫画 ID
    '''
    # 初始化
    def __init__(self, id: int) -> None:
        '''漫画类

        参数:
            id (int): 漫画 ID
        '''
        self.id: int = id
        '''漫画 ID'''
        self.__info: Dict[str, Any] = {}
        '''漫画信息'''
        return
    
    # 漫画标题
    @property
    def title(self) -> str:
        '''漫画标题'''
        info = self.__info
        # 获取 title 数据
        if (title := info.get('title', None)) is None:
            raise NoDataException('漫画标题')
        # 获取第一个非 None 漫画标题
        try:
            return next(filter(lambda x: x is not None, title))
        except StopIteration:
            raise NoDataException('漫画标题')
    
    # 漫画副标题
    @property
    def sub_title(self) -> str:
        '''漫画副标题'''
        info = self.__info
        # 获取 subTitle 数据
        if (sub_title := info.get('subTitle', None)) is None:
            raise NoDataException('漫画副标题')
        # 获取第一个非 None 漫画副标题
        try:
            return next(filter(lambda x: x is not None, sub_title))
        except StopIteration:
            raise NoDataException('漫画副标题')
    
    # 漫画默认服务器
    @property
    def server(self) -> Literal['jp', 'en', 'tw', 'cn', 'kr']:
        '''漫画默认服务器

        返回:
            Literal[&#39;jp&#39;, &#39;en&#39;, &#39;tw&#39;, &#39;cn&#39;, &#39;kr&#39;]: 歌曲所在服务器
        '''
        info = self.__info
        # 获取 publicStartAt 数据
        if (public_start_at := info.get('publicStartAt', None)) is None:
            raise NoDataException('漫画 ID 列表')
        # 根据 publicStartAt 数据判断服务器
        if public_start_at[0] is not None: return 'jp'
        elif public_start_at[1] is not None: return 'en'
        elif public_start_at[2] is not None: return 'tw'
        elif public_start_at[3] is not None: return 'cn'
        elif public_start_at[4] is not None: return 'kr'
        else:
            raise NoDataException('漫画所在服务器')
    
    # 漫画类型
    @property
    def type(self) -> Literal['singleframe', 'fourframe']:
        '''漫画类型

        返回:
            Literal[&#39;singleframe&#39;, &#39;fourframe&#39;]: 漫画类型
        '''
        # 获取漫画数据包名称
        info = self.__info
        if (asset_bundle_name := info.get('assetBundleName', None)) is None:
            raise ValueError('无法获取漫画数据包名称。')
        # 判断漫画类型
        if 'fourframe' in asset_bundle_name:
            return 'fourframe'
        else:
            return 'singleframe'
    
    # 获取漫画信息
    def get_info(self) -> Dict[str, Any]:
        '''获取漫画信息

        返回:
            Dict[str, Any]: 漫画详细信息
        '''
        if not self.__info:
            _all = get_all()
            if str(self.id) not in _all:
                raise ComicNotExistError(self.id)
            self.__info = _all[str(self.id)]
        return self.__info
    
    # 异步获取漫画信息
    async def get_info_async(self) -> Dict[str, Any]:
        '''获取漫画信息

        返回:
            Dict[str, Any]: 漫画详细信息
        '''
        if not self.__info:
            _all = await get_all_async()
            if str(self.id) not in _all:
                raise ComicNotExistError(self.id)
            self.__info = _all[str(self.id)]
        return self.__info
    
    # 获取缓存信息
    def __get_info_cache(self) -> Dict[str, Any]:
        '''获取缓存信息

        返回:
            Dict[str, Any]: 缓存信息
        '''
        if not self.__info:
            return self.get_info()
        return self.__info
    
    # 异步获取缓存信息
    async def __get_info_cache_async(self) -> Dict[str, Any]:
        '''获取缓存信息

        返回:
            Dict[str, Any]: 缓存信息
        '''
        if not self.__info:
            return await self.get_info_async()
        return self.__info
    
    # 获取漫画评论
    def get_comment(
        self,
        limit: int=20,
        offset: int=0,
        order: Literal['TIME_DESC', 'TIME_ASC']='TIME_ASC'
    ) -> Dict[str, Any]:
        '''获取漫画评论

        参数:
            limit (int, optional): 展示出的评论数，默认为 20
            offset (int, optional): 忽略前面的 `offset` 条评论，默认为 0
            order (Literal[&#39;TIME_DESC&#39;, &#39;TIME_ASC&#39;], optional): 排序顺序，默认时间顺序

        返回:
            Dict[str, Any]: 搜索结果
                ```python
                {
                    "result": ... # bool 是否有响应
                    "count": ... # int 搜索到的评论总数
                    "posts": ... # List[Dict[str, Any]] 列举出的评论
                }
                ```
        '''
        return get_list(
            category_name='COMIC_COMMENT',
            category_id=str(self.id),
            order=order,
            limit=limit,
            offset=offset
        )
    
    # 异步获取漫画评论
    async def get_comment_async(
        self,
        limit: int=20,
        offset: int=0,
        order: Literal['TIME_DESC', 'TIME_ASC']='TIME_ASC'
    ) -> Dict[str, Any]:
        '''获取漫画评论

        参数:
            limit (int, optional): 展示出的评论数，默认为 20
            offset (int, optional): 忽略前面的 `offset` 条评论，默认为 0
            order (Literal[&#39;TIME_DESC&#39;, &#39;TIME_ASC&#39;], optional): 排序顺序，默认时间顺序

        返回:
            Dict[str, Any]: 搜索结果
                ```python
                {
                    "result": ... # bool 是否有响应
                    "count": ... # int 搜索到的评论总数
                    "posts": ... # List[Dict[str, Any]] 列举出的评论
                }
                ```
        '''
        return await get_list_async(
            category_name='COMIC_COMMENT',
            category_id=str(self.id),
            order=order,
            limit=limit,
            offset=offset
        )
    
    # 获取漫画缩略图图像
    def get_thumbnail(self, server: Literal['jp', 'en', 'tw', 'cn', 'kr']) -> bytes:
        '''获取漫画缩略图图像

        参数:
            server (Literal[&#39;jp&#39;, &#39;en&#39;, &#39;tw&#39;, &#39;cn&#39;, &#39;kr&#39;]): 指定服务器

        返回:
            bytes: 漫画缩略图图像字节数据 `bytes`
        '''
        # 获取漫画数据包名称
        info = self.__get_info_cache()
        if (asset_bundle_name := info.get('assetBundleName', None)) is None:
            raise ValueError('无法获取漫画数据包名称。')
        # 判断服务器
        if (public_start_at := info.get('publicStartAt', None)) is None:
            raise ValueError('无法获取漫画起始时间。')
        SERVERS = ['jp', 'en', 'tw', 'cn', 'kr']
        index = SERVERS.index(server)
        if public_start_at[index] is None:
            raise ServerNotAvailableError(f'漫画 {self.title}', server)
        return Assets(
            ASSETS['comic']['thumbnail'].format(
                type=self.type, asset_bundle_name=asset_bundle_name
            ), server
        ).get()
    
    # 异步获取漫画缩略图图像
    async def get_thumbnail_async(self, server: Literal['jp', 'en', 'tw', 'cn', 'kr']) -> bytes:
        '''获取漫画缩略图图像

        参数:
            server (Literal[&#39;jp&#39;, &#39;en&#39;, &#39;tw&#39;, &#39;cn&#39;, &#39;kr&#39;]): 指定服务器

        返回:
            bytes: 漫画缩略图图像字节数据 `bytes`
        '''
        # 获取漫画数据包名称
        info = await self.__get_info_cache_async()
        if (asset_bundle_name := info.get('assetBundleName', None)) is None:
            raise ValueError('无法获取漫画数据包名称。')
        # 判断服务器
        if (public_start_at := info.get('publicStartAt', None)) is None:
            raise ValueError('无法获取漫画起始时间。')
        SERVERS = ['jp', 'en', 'tw', 'cn', 'kr']
        index = SERVERS.index(server)
        if public_start_at[index] is None:
            raise ServerNotAvailableError(f'漫画 {self.title}', server)
        return await Assets(
            ASSETS['comic']['thumbnail'].format(
                type=self.type, asset_bundle_name=asset_bundle_name
            ), server
        ).aget()
    
    # 获取漫画图像
    def get(self, server: Literal['jp', 'en', 'tw', 'cn', 'kr']) -> bytes:
        '''获取漫画图像

        参数:
            server (Literal[&#39;jp&#39;, &#39;en&#39;, &#39;tw&#39;, &#39;cn&#39;, &#39;kr&#39;]): 指定服务器

        返回:
            bytes: 漫画图像字节数据 `bytes`
        '''
        # 获取漫画数据包名称
        info = self.__get_info_cache()
        if (asset_bundle_name := info.get('assetBundleName', None)) is None:
            raise ValueError('无法获取漫画数据包名称。')
        # 判断服务器
        if (public_start_at := info.get('publicStartAt', None)) is None:
            raise ValueError('无法获取漫画起始时间。')
        SERVERS = ['jp', 'en', 'tw', 'cn', 'kr']
        index = SERVERS.index(server)
        if public_start_at[index] is None:
            raise ServerNotAvailableError(f'漫画 {self.title}', server)
        return Assets(
            ASSETS['comic']['comic'].format(
                type=self.type, asset_bundle_name=asset_bundle_name
            ), server
        ).get()
    
    # 异步获取漫画图像
    async def aget(self, server: Literal['jp', 'en', 'tw', 'cn', 'kr']) -> bytes:
        '''获取漫画图像

        参数:
            server (Literal[&#39;jp&#39;, &#39;en&#39;, &#39;tw&#39;, &#39;cn&#39;, &#39;kr&#39;]): 指定服务器

        返回:
            bytes: 漫画图像字节数据 `bytes`
        '''
        # 获取漫画数据包名称
        info = await self.__get_info_cache_async()
        if (asset_bundle_name := info.get('assetBundleName', None)) is None:
            raise ValueError('无法获取漫画数据包名称。')
        # 判断服务器
        if (public_start_at := info.get('publicStartAt', None)) is None:
            raise ValueError('无法获取漫画起始时间。')
        SERVERS = ['jp', 'en', 'tw', 'cn', 'kr']
        index = SERVERS.index(server)
        if public_start_at[index] is None:
            raise ServerNotAvailableError(f'漫画 {self.title}', server)
        return await Assets(
            ASSETS['comic']['comic'].format(
                type=self.type, asset_bundle_name=asset_bundle_name
            ), server
        ).aget()



'''`bestdori.missions`

BanG Dream! 任务相关操作'''
from typing import Any, Dict, List, Literal

from aiohttp import ClientResponseError
from httpx import Response, HTTPStatusError

from .utils.utils import API
from .utils.network import Api
from .post import get_list, get_list_async
from .exceptions import (
    NoDataException,
    MissionNotExistError
)

# 获取总任务信息
def get_all(index: Literal[0, 5]=5) -> Dict[str, Dict[str, Any]]:
    '''获取总任务信息

    参数:
        index (Literal[0, 5], optional): 指定获取哪种 `all.json`
            `0`: 仅获取所有已有任务 ID `all.0.json`
            `5`: 获取所有已有任务的简洁信息 `all.5.json`

    返回:
        Dict[str, Dict[str, Any]]: 获取到的总任务信息
    '''
    return Api(API['missions']['all'].format(index=index)).get().json()

# 异步获取总任务信息
async def get_all_async(index: Literal[0, 5]=5) -> Dict[str, Dict[str, Any]]:
    '''获取总任务信息

    参数:
        index (Literal[0, 5], optional): 指定获取哪种 `all.json`
            `0`: 仅获取所有已有任务 ID `all.0.json`
            `5`: 获取所有已有任务的简洁信息 `all.5.json`

    返回:
        Dict[str, Dict[str, Any]]: 获取到的总任务信息
    '''
    response = await Api(API['missions']['all'].format(index=index)).aget()
    if isinstance(response, Response): return response.json()
    return await response.json()

# 任务类
class Mission:
    '''任务类

    参数:
        id (int): 任务 ID
    '''
    # 初始化
    def __init__(self, id: int) -> None:
        '''任务类

        参数:
            id (int): 任务 ID
        '''
        self.id: int = id
        '''任务 ID'''
        self.__info: Dict[str, Any] = {}
        '''任务信息'''
        return
    
    # 任务标题
    @property
    def title(self) -> str:
        '''任务标题'''
        info = self.__info
        # 获取 title 数据
        if (title := info.get('title', None)) is None:
            raise NoDataException('任务标题')
        # 获取第一个非 None 任务标题
        try:
            return next(filter(lambda x: x is not None, title))
        except StopIteration:
            raise NoDataException('任务标题')
    
    # 任务所在默认服务器
    @property
    def server(self) -> Literal['jp', 'en', 'tw', 'cn', 'kr']:
        '''任务所在默认服务器'''
        info = self.__info
        # 获取 startAt 数据
        if (start_at := info.get('startAt', None)) is None:
            raise NoDataException('任务起始时间')
        # 根据 startAt 数据判断服务器
        if start_at[0] is not None: return 'jp'
        elif start_at[1] is not None: return 'en'
        elif start_at[2] is not None: return 'tw'
        elif start_at[3] is not None: return 'cn'
        elif start_at[4] is not None: return 'kr'
        else:
            raise NoDataException('任务所在服务器')
    
    # 获取任务信息
    def get_info(self) -> Dict[str, Any]:
        '''获取任务信息

        返回:
            Dict[str, Any]: 任务详细信息
        '''
        try:
            response = Api(
                API['missions']['info'].format(id=self.id)
            ).get()
        except HTTPStatusError as exception:
            if exception.response.status_code == 404:
                raise MissionNotExistError(self.id)
            raise exception
        
        self.__info = dict(response.json())
        return self.__info
    
    # 异步获取任务信息
    async def get_info_async(self) -> Dict[str, Any]:
        '''获取任务信息

        返回:
            Dict[str, Any]: 任务详细信息
        '''
        try:
            response = await Api(
                API['missions']['info'].format(id=self.id)
            ).aget()
        except HTTPStatusError as exception:
            if exception.response.status_code == 404:
                raise MissionNotExistError(self.id)
            raise exception
        except ClientResponseError as exception:
            if exception.status == 404:
                raise MissionNotExistError(self.id)
            raise exception
        
        if isinstance(response, Response):
            self.__info = dict(response.json())
        else:
            self.__info = dict(await response.json())
        return self.__info
    
    # 获取任务评论
    def get_comment(
        self,
        limit: int=20,
        offset: int=0,
        order: Literal['TIME_DESC', 'TIME_ASC']='TIME_ASC'
    ) -> Dict[str, Any]:
        '''获取任务评论

        参数:
            limit (int, optional): 展示出的评论数，默认为 20
            offset (int, optional): 忽略前面的 `offset` 条评论，默认为 0
            order (Literal[&#39;TIME_DESC&#39;, &#39;TIME_ASC&#39;], optional): 排序顺序，默认时间顺序

        返回:
            Dict[str, Any]: 搜索结果
                ```python
                {
                    "result": ... # bool 是否有响应
                    "count": ... # int 搜索到的评论总数
                    "posts": ... # List[Dict[str, Any]] 列举出的评论
                }
                ```
        '''
        return get_list(
            category_name='MISSION_COMMENT',
            category_id=str(self.id),
            order=order,
            limit=limit,
            offset=offset
        )
    
    # 异步获取任务评论
    async def get_comment_async(
        self,
        limit: int=20,
        offset: int=0,
        order: Literal['TIME_DESC', 'TIME_ASC']='TIME_ASC'
    ) -> Dict[str, Any]:
        '''获取任务评论

        参数:
            limit (int, optional): 展示出的评论数，默认为 20
            offset (int, optional): 忽略前面的 `offset` 条评论，默认为 0
            order (Literal[&#39;TIME_DESC&#39;, &#39;TIME_ASC&#39;], optional): 排序顺序，默认时间顺序

        返回:
            Dict[str, Any]: 搜索结果
                ```python
                {
                    "result": ... # bool 是否有响应
                    "count": ... # int 搜索到的评论总数
                    "posts": ... # List[Dict[str, Any]] 列举出的评论
                }
                ```
        '''
        return await get_list_async(
            category_name='MISSION_COMMENT',
            category_id=str(self.id),
            order=order,
            limit=limit,
            offset=offset
        )
    


'''`bestdori.utils.content`

Bestdori 帖子内容模块'''
from typing import Literal, Any, Dict, List

# 内容类
class Content:
    '''内容类'''
    type: str
    '''内容类型'''
    # 初始化
    def __init__(self, values: Dict[str, Any]) -> None:
        '''初始化'''
        for key, value in values.items():
            setattr(self, key, value)
        return
    
    # 纯文本
    @staticmethod
    def text(data: str) -> 'Text':
        '''纯文本

        参数:
            data (str): 文本内容

        返回:
            Text: 文本对象 `bestdori.utils.Text`
        '''
        return Text({'type': 'text', 'data': data})
    
    # 换行
    @staticmethod
    def br() -> 'Br':
        '''换行

        返回:
            Br: 换行对象 `bestdori.utils.Br`
        '''
        return Br({'type': 'br'})
    
    # 表情
    @staticmethod
    def emoji(data: str) -> 'Emoji':
        '''表情

        参数:
            data (str): 表情名称

        返回:
            Emoji: 表情对象 `bestdori.utils.Emoji`
        '''
        return Emoji({'type': 'emoji', 'data': data})
    
    # 提及
    @staticmethod
    def mention(data: str) -> 'Mention':
        '''提及

        参数:
            data (str): 提及的用户名

        返回:
            Mention: 提及对象 `bestdori.utils.Mention`
        '''
        return Mention({'type': 'mention', 'data': data})
    
    # 标题
    @staticmethod
    def heading(data: str, margin: Literal['top']='top') -> 'Heading':
        '''标题

        参数:
            data (str): 标题内容
            
            margin (Literal[&#39;top&#39;], optional): 页边空白位置

        返回:
            Heading: 标题对象 `bestdori.utils.Heading`
        '''
        return Heading({'type': 'heading', 'data': data, 'margin': margin})
    
    # 图片
    @staticmethod
    def image(objects: List[str], display: Literal[0, 1, 2]=0) -> 'Image':
        '''图片

        参数:
            objects (List[str]): 图片对象网址列表
            
            display (Literal[&#39;0&#39;, &#39;1&#39;, &#39;2&#39;], optional): 显示类型 `0`: 大图 `1`: 缩略图 `2`: 图标

        返回:
            Image: 图片对象 `bestdori.utils.Image`
        '''
        return Image({'type': 'image', 'objects': objects, 'display': display})
    
    # 链接
    @staticmethod
    def link(
        target: Literal[
            'url',
            'character-single',
            'card-single',
            'costume-single',
            'event-single',
            'gacha-single',
            'song-single',
            'logincampaign-single',
            'comic-single',
            'mission-single'
        ],
        data: str
    ) -> 'Link':
        '''链接

        参数:
            target (str): 链接对象
            
            data (str): 链接信息

        返回:
            Link: 链接对象 `bestdori.utils.Link`
        '''
        if target != 'url':
            if not data.isdigit():
                raise ValueError('非 url 链接对象的 data 必须为数字。')
        return Link({'type': 'link', 'target': target, 'data': data})
    
    # 列表
    @staticmethod
    def list(
        target: Literal[
            'character-info',
            'card-info',
            'card-icon',
            'costume-info',
            'event-info',
            'gacha-info',
            'song-info',
            'logincampaign-info',
            'comic-info',
            'mission-info'
        ],
        display: Literal[0, 1, 2],
        objects: List[str]
    ) -> 'ContentList':
        '''列表

        参数:
            target (str): 列表对象
            
            display (Literal[&#39;0&#39;, &#39;1&#39;, &#39;2&#39;]): 显示类型
            
            objects (list[str]): 列表对象 ID 列表

        返回:
            List: 列表对象 `bestdori.utils.List`
        '''
        return ContentList({'type': 'list', 'target': target, 'display': display, 'objects': objects})

# 文本类
class Text(Content):
    '''文本类'''
    type: str = 'text'
    '''内容类型'''
    data: str
    '''文本内容'''

# 换行类
class Br(Content):
    type: str = 'br'
    '''内容类型'''

# 表情类
class Emoji(Content):
    '''表情类'''
    type: str = 'emoji'
    '''内容类型'''
    data: str
    '''表情名称'''

# 提及类
class Mention(Content):
    '''提及类'''
    type: str = 'mention'
    '''内容类型'''
    data: str
    '''提及的用户名'''

# 标题类
class Heading(Content):
    '''标题类'''
    type: str = 'heading'
    '''内容类型'''
    data: str
    '''标题内容'''
    margin: str
    '''页边空白位置'''

# 图片类
class Image(Content):
    '''图片类'''
    type: str = 'image'
    '''内容类型'''
    display: Literal[0, 1, 2]
    '''显示类型 `0`: 大图 `1`: 缩略图 `2`: 图标'''
    objects: List[str]
    '''图片对象网址列表'''

# 链接类
class Link(Content):
    '''链接类'''
    type: str = 'link'
    '''内容类型'''
    target: str
    '''链接对象'''
    data: str
    '''链接信息'''

# 列表类
class ContentList(Content):
    '''列表类'''
    type: str = 'list'
    '''内容类型'''
    target: str
    '''列表对象'''
    display: Literal[0, 1, 2]
    '''显示类型'''
    objects: List[str]
    '''列表对象 ID 列表'''



'''`bestdori.costumes`

BanG Dream! 服装相关操作'''
from typing import Any, Dict, Literal

from aiohttp import ClientResponseError
from httpx import Response, HTTPStatusError

from .utils.utils import API, ASSETS
from .utils.network import Api, Assets
from .post import get_list, get_list_async
from .exceptions import (
    NoDataException,
    AssetsNotExistError,
    CostumeNotExistError
)

# 获取总服装信息
def get_all(index: Literal[0, 5]=5) -> Dict[str, Dict[str, Any]]:
    '''获取总服装信息

    参数:
        index (Literal[0, 5], optional): 指定获取哪种 `all.json`
            `0`: 仅获取所有已有服装 ID `all.0.json`
            `5`: 获取所有已有服装的简洁信息 `all.5.json`

    返回:
        Dict[str, Dict[str, Any]]: 获取到的总服装信息
    '''
    return Api(API['costumes']['all'].format(index=index)).get().json()

# 异步获取总服装信息
async def get_all_async(index: Literal[0, 5]=5) -> Dict[str, Dict[str, Any]]:
    '''获取总服装信息

    参数:
        index (Literal[0, 5], optional): 指定获取哪种 `all.json`
            `0`: 仅获取所有已有服装 ID `all.0.json`
            `5`: 获取所有已有服装的简洁信息 `all.5.json`

    返回:
        Dict[str, Dict[str, Any]]: 获取到的总服装信息
    '''
    response = await Api(API['costumes']['all'].format(index=index)).aget()
    if isinstance(response, Response):
        return response.json()
    else:
        return await response.json()

# 服装类
class Costume:
    '''服装类

    参数:
        id (int): 服装 ID
    '''
    # 初始化
    def __init__(self, id: int) -> None:
        '''服装类

        参数:
            id (int): 服装 ID
        '''
        self.id: int = id
        '''服装 ID'''
        self.__info: Dict[str, Any] = {}
        '''服装信息'''
        return
    
    # 角色 ID
    @property
    def character_id(self) -> int:
        '''角色 ID'''
        info = self.__info
        # 获取 characterId 数据
        if (character_id := info.get('characterId', None)) is None:
            raise NoDataException('角色 ID')
        return character_id
    
    # 卡牌 ID
    @property
    def card_id(self) -> int:
        '''卡牌 ID'''
        info = self.__info
        # 获取 cards 数据
        if (cards := info.get('cards', None)) is None:
            raise NoDataException('卡牌 ID')
        SERVERS = ['jp', 'en', 'tw', 'cn', 'kr']
        index = SERVERS.index(self.server)
        return cards[index]
    
    # 服装标题
    @property
    def description(self) -> str:
        '''服装标题'''
        info = self.__info
        # 获取 description 数据
        if (description := info.get('description', None)) is None:
            raise NoDataException('服装标题')
        # 获取第一个非 None 服装标题
        try:
            return next(filter(lambda x: x is not None, description))
        except StopIteration:
            raise NoDataException('服装标题')
    
    # 服装所在默认服务器
    @property
    def server(self) -> Literal['jp', 'en', 'tw', 'cn', 'kr']:
        '''服装所在默认服务器'''
        info = self.__info
        # 获取 publishedAt 数据
        if (published_at := info.get('publishedAt', None)) is None:
            raise NoDataException('服装发布时间')
        # 根据 publishedAt 数据判断服务器
        if published_at[0] is not None: return 'jp'
        elif published_at[1] is not None: return 'en'
        elif published_at[2] is not None: return 'tw'
        elif published_at[3] is not None: return 'cn'
        elif published_at[4] is not None: return 'kr'
        else:
            raise NoDataException('服装所在服务器')
    
    # 获取服装信息
    def get_info(self) -> Dict[str, Any]:
        '''获取服装信息

        返回:
            Dict[str, Any]: 服装详细信息
        '''
        try:
            response = Api(
                API['costumes']['info'].format(id=self.id)
            ).get()
        except HTTPStatusError as exception:
            if exception.response.status_code == 404:
                raise CostumeNotExistError(self.id)
            else:
                raise exception
        
        self.__info = dict(response.json())
        return self.__info
    
    # 异步获取服装信息
    async def get_info_async(self) -> Dict[str, Any]:
        '''获取服装信息

        返回:
            Dict[str, Any]: 服装详细信息
        '''
        try:
            response = await Api(
                API['costumes']['info'].format(id=self.id)
            ).aget()
        except HTTPStatusError as exception:
            if exception.response.status_code == 404:
                raise CostumeNotExistError(self.id)
            else:
                raise exception
        except ClientResponseError as exception:
            if exception.status == 404:
                raise CostumeNotExistError(self.id)
            else:
                raise exception
        
        if isinstance(response, Response):
            self.__info = dict(response.json())
        else:
            self.__info = dict(await response.json())
        return self.__info
    
    # 获取缓存信息
    def __get_info_cache(self) -> Dict[str, Any]:
        '''获取缓存信息

        返回:
            Dict[str, Any]: 服装详细信息
        '''
        if not self.__info:
            return self.get_info()
        return self.__info
    
    # 异步获取缓存信息
    async def __get_info_cache_async(self) -> Dict[str, Any]:
        '''获取缓存信息

        返回:
            Dict[str, Any]: 服装详细信息
        '''
        if not self.__info:
            return await self.get_info_async()
        return self.__info
    
    # 获取服装评论
    def get_comment(
        self,
        limit: int=20,
        offset: int=0,
        order: Literal['TIME_DESC', 'TIME_ASC']='TIME_ASC'
    ) -> Dict[str, Any]:
        '''获取服装评论

        参数:
            limit (int, optional): 展示出的评论数，默认为 20
            offset (int, optional): 忽略前面的 `offset` 条评论，默认为 0
            order (Literal[&#39;TIME_DESC&#39;, &#39;TIME_ASC&#39;], optional): 排序顺序，默认时间顺序

        返回:
            Dict[str, Any]: 搜索结果
                ```python
                {
                    "result": ... # bool 是否有响应
                    "count": ... # int 搜索到的评论总数
                    "posts": ... # List[Dict[str, Any]] 列举出的评论
                }
                ```
        '''
        return get_list(
            category_name='COSTUME_COMMENT',
            category_id=str(self.id),
            order=order,
            limit=limit,
            offset=offset
        )
    
    # 异步获取服装评论
    async def get_comment_async(
        self,
        limit: int=20,
        offset: int=0,
        order: Literal['TIME_DESC', 'TIME_ASC']='TIME_ASC'
    ) -> Dict[str, Any]:
        '''获取服装评论

        参数:
            limit (int, optional): 展示出的评论数，默认为 20
            offset (int, optional): 忽略前面的 `offset` 条评论，默认为 0
            order (Literal[&#39;TIME_DESC&#39;, &#39;TIME_ASC&#39;], optional): 排序顺序，默认时间顺序

        返回:
            Dict[str, Any]: 搜索结果
                ```python
                {
                    "result": ... # bool 是否有响应
                    "count": ... # int 搜索到的评论总数
                    "posts": ... # List[Dict[str, Any]] 列举出的评论
                }
                ```
        '''
        return await get_list_async(
            category_name='COSTUME_COMMENT',
            category_id=str(self.id),
            order=order,
            limit=limit,
            offset=offset
        )
    
    # 获取 LIVE 服装图片
    def get_sdchara(self) -> bytes:
        '''获取 LIVE 服装图片

        返回:
            bytes: 服装 LIVE 图片字节数据 `bytes`
        '''
        # 获取服装 sdchara 数据包名称
        info = self.__get_info_cache()
        if (sd_resource_name := info.get('sdResourceName', None)) is None:
            raise ValueError('无法获取服装数据包名称。')
        return Assets(
            ASSETS['characters']['livesd'].format(
                sd_resource_name=sd_resource_name
            ), self.server
        ).get()
    
    # 异步获取 LIVE 服装图片
    async def get_sdchara_async(self) -> bytes:
        '''获取 LIVE 服装图片

        返回:
            bytes: 服装 LIVE 图片字节数据 `bytes`
        '''
        # 获取服装 sdchara 数据包名称
        info = await self.__get_info_cache_async()
        if (sd_resource_name := info.get('sdResourceName', None)) is None:
            raise ValueError('无法获取服装数据包名称。')
        return await Assets(
            ASSETS['characters']['livesd'].format(
                sd_resource_name=sd_resource_name
            ), self.server
        ).aget()
    
    # 获取服装模型数据
    def get_build_data(self) -> bytes:
        '''获取服装模型数据

        返回:
            bytes: 服装模型数据
        '''
        # 获取服装数据包名称
        info = self.__get_info_cache()
        if (asset_bundle_name := info.get('assetBundleName', None)) is None:
            raise ValueError('无法获取服装数据包名称。')
        try:
            return Assets(
                ASSETS['live2d']['buildData'].format(
                    asset_bundle_name=asset_bundle_name
                ), self.server
            ).get()
        except AssetsNotExistError:
            raise AssetsNotExistError(f'服装模型 {asset_bundle_name}-{self.server}')
    
    # 异步获取服装模型数据
    async def get_build_data_async(self) -> bytes:
        '''获取服装模型数据

        返回:
            bytes: 服装模型数据
        '''
        # 获取服装数据包名称
        info = await self.__get_info_cache_async()
        if (asset_bundle_name := info.get('assetBundleName', None)) is None:
            raise ValueError('无法获取服装数据包名称。')
        try:
            return await Assets(
                ASSETS['live2d']['buildData'].format(
                    asset_bundle_name=asset_bundle_name
                ), self.server
            ).aget()
        except AssetsNotExistError:
            raise AssetsNotExistError(f'服装模型 {asset_bundle_name}-{self.server}')
    
    # 获取服装图标
    def get_icon(self) -> bytes:
        '''获取服装图标

        返回:
            bytes: 服装图标
        '''
        # 获取服装数据包名称
        info = self.__get_info_cache()
        if (asset_bundle_name := info.get('assetBundleName', None)) is None:
            raise ValueError('无法获取服装数据包名称。')
        try:
            return Assets(
                ASSETS['thumb']['costume'].format(
                    id=str(self.id // 50), asset_bundle_name=asset_bundle_name
                ), self.server
            ).get()
        except AssetsNotExistError:
            raise AssetsNotExistError(f'服装图标 {asset_bundle_name}-{self.server}')
    
    # 异步获取服装图标
    async def get_icon_async(self) -> bytes:
        '''获取服装图标

        返回:
            bytes: 服装图标
        '''
        # 获取服装数据包名称
        info = await self.__get_info_cache_async()
        if (asset_bundle_name := info.get('assetBundleName', None)) is None:
            raise ValueError('无法获取服装数据包名称。')
        try:
            return await Assets(
                ASSETS['thumb']['costume'].format(
                    id=str(self.id // 50), asset_bundle_name=asset_bundle_name
                ), self.server
            ).aget()
        except AssetsNotExistError:
            raise AssetsNotExistError(f'服装图标 {asset_bundle_name}-{self.server}')



'''`bestdori.upload`

Bestdori 文件相关操作'''
from typing import Union
from pathlib import Path
from hashlib import sha1
from io import BufferedReader

from httpx import Response

from .user import Me
from . import settings
from .utils.utils import API
from .utils.network import Api
from .exceptions import AlreadyUploadedError

# 从 Bestdori 获取指定哈希文件字节
def download(hash: str) -> bytes:
    '''从 Bestdori 获取指定哈希文件字节

    参数:
        hash (str): 文件哈希值

    返回:
        bytes: 文件字节 `bytes`
    '''
    return Api(API['upload']['file'].format(hash=hash)).get().content

# 异步从 Bestdori 获取指定哈希文件字节
async def adownload(hash: str) -> bytes:
    '''从 Bestdori 获取指定哈希文件字节

    参数:
        hash (str): 文件哈希值

    返回:
        bytes: 文件字节 `bytes`
    '''
    response = await Api(API['upload']['file'].format(hash=hash)).aget()
    if isinstance(response, Response): return response.content
    return await response.read()

# 通过哈希值构建 Bestdori 文件 URL
def hash_to_url(hash: str) -> str:
    '''通过哈希值构建 Bestdori 文件 URL

    参数:
        hash (str): 文件哈希值

    返回:
        str: 文件 URL
    '''
    return f'https://bestdori.com/api/upload/file/{hash}'

# 上传文件类
class Upload:
    '''上传文件类

    参数:
        file_bytes (bytes): 文件字节
        name (str): 文件名
        reader (BufferedReader): 文件字节流
    '''
    # 初始化
    def __init__(self, file_bytes: bytes, name: str, reader: BufferedReader) -> None:
        # 处理文件字节
        hash = sha1(file_bytes).hexdigest() # 计算 SHA-1 哈希
        size = len(file_bytes) # 计算文件大小
        # 赋值对象属性
        self._ver: int = 3
        '''文件版本'''
        self._hash: str = hash
        '''文件哈希'''
        self._size: int = size
        '''文件大小'''
        self._name: str = name
        '''文件名'''
        self._reader: BufferedReader = reader
        '''文件字节流'''
        return
    
    # 从路径中获取文件
    @classmethod
    def from_path(cls, path: Union[str, Path]) -> 'Upload':
        '''从路径中获取文件

        参数:
            path (Union[str, Path]): 文件路径

        返回:
            Upload: 上传文件对象
        '''
        # 从路径中获取文件名
        path = Path(path)
        file_name = path.name
        # 读取文件字节
        file = open(path, 'rb')
        file_bytes = file.read()
        # 返回上传文件对象
        return cls(file_bytes, file_name, file)
    
    # 上传文件
    def upload(self) -> str:
        '''上传文件

        返回:
            str: 上传文件的哈希值
        '''
        try:
            cookies = settings._cookies()
        except:
            if settings._username is None or settings._password is None:
                raise ValueError('未添加用户信息。')
            else:
                settings._me = Me.login(settings._username, settings._password)
                cookies = settings._cookies()
        
        if self._reader.closed:
            raise ValueError('文件流已关闭。')
        
        # 构建上传负载
        payload = {
            'ver': self._ver,
            'hash': self._hash,
            'size': self._size
        }
        # 发送预上传请求
        try:
            Api(API['upload']['prepare']).post(cookies=cookies, data=payload)
        except AlreadyUploadedError:
            self._reader.close()
            return self._hash
        # 发送上传请求
        with self._reader:
            response = Api(API['upload']['upload']).post(
                files={
                    'file': (self._name, self._reader)
                },
                cookies=cookies
            )
        # 获取文件的哈希值
        hash_get = response.json()['hash']
        #重复查询至多 5 次上传状态
        for _ in range(5):
            # 发送上传状态查询请求
            response = Api(API['upload']['status'].format(hash=hash_get)).get()
            # 获取上传状态
            status = response.json()['status']
            # 若上传成功则返回
            if status == 'available':
                return hash_get
        
        raise TimeoutError(f'上传文件 {self._name} 超时。')
    
    # 异步上传文件
    async def aupload(self) -> str:
        '''上传文件

        返回:
            str: 上传文件的哈希值
        '''
        try:
            cookies = settings._cookies()
        except:
            if settings._username is None or settings._password is None:
                raise ValueError('未添加用户信息。')
            else:
                settings._me = await Me.alogin(settings._username, settings._password)
                cookies = settings._cookies()
        
        if self._reader.closed:
            raise ValueError('文件流已关闭。')
        
        # 构建上传负载
        payload = {
            'ver': self._ver,
            'hash': self._hash,
            'size': self._size
        }
        # 发送预上传请求
        try:
            await Api(API['upload']['prepare']).apost(cookies=cookies, data=payload)
        except AlreadyUploadedError:
            self._reader.close()
            return self._hash
        # 发送上传请求
        with self._reader:
            response = await Api(API['upload']['upload']).apost(
                files={
                    'file': (self._name, self._reader)
                },
                cookies=cookies
            )
        # 获取文件的哈希值
        if isinstance(response, Response):
            hash_get = response.json()['hash']
        else:
            hash_get = (await response.json())['hash']
        #重复查询至多 5 次上传状态
        for _ in range(5):
            # 发送上传状态查询请求
            response = await Api(API['upload']['status'].format(hash=hash_get)).aget()
            # 获取上传状态
            if isinstance(response, Response):
                status = response.json()['status']
            else:
                status = (await response.json())['status']
            # 若上传成功则返回
            if status == 'available':
                return hash_get
        
        raise TimeoutError(f'上传文件 {self._name} 超时。')



'''`bestdori.gacha`

BanG Dream! 招募相关操作'''
import asyncio
from typing import Any, Dict, List, Literal

from aiohttp import ClientResponseError
from httpx import Response, HTTPStatusError

from .utils.utils import API, ASSETS
from .utils.network import Api, Assets
from .post import get_list, get_list_async
from .exceptions import (
    NoDataException,
    GachaNotExistError,
    AssetsNotExistError,
    ServerNotAvailableError
)

# 获取总招募信息
def get_all(index: Literal[0, 5]=5) -> Dict[str, Dict[str, Any]]:
    '''获取总招募信息

    参数:
        index (Literal[0, 5], optional): 指定获取哪种 `all.json`
            `0`: 仅获取所有已有招募 ID `all.0.json`
            `5`: 获取所有已有招募的简洁信息 `all.5.json`

    返回:
        Dict[str, Dict[str, Any]]: 获取到的总招募信息
    '''
    return Api(API['gacha']['all'].format(index=index)).get().json()

# 异步获取总招募信息
async def get_all_async(index: Literal[0, 5]=5) -> Dict[str, Dict[str, Any]]:
    '''获取总招募信息

    参数:
        index (Literal[0, 5], optional): 指定获取哪种 `all.json`
            `0`: 仅获取所有已有招募 ID `all.0.json`
            `5`: 获取所有已有招募的简洁信息 `all.5.json`

    返回:
        Dict[str, Dict[str, Any]]: 获取到的总招募信息
    '''
    response = await Api(API['gacha']['all'].format(index=index)).aget()
    if isinstance(response, Response):
        return response.json()
    else:
        return await response.json()

# 招募类
class Gacha:
    '''招募类

    参数:
        id (int): 招募 ID
    '''
    # 初始化
    def __init__(self, id: int) -> None:
        '''招募类

        参数:
            id (int): 招募 ID
        '''
        self.id: int = id
        '''招募 ID'''
        self.__info: Dict[str, Any] = {}
        '''招募信息'''
        return
    
    # 获取招募标题
    @property
    def name(self) -> str:
        '''获取招募标题

        返回:
            str: 招募标题
        '''
        info = self.__info
        # 获取 eventName 数据
        if (gacha_name := info.get('gachaName', None)) is None:
            raise NoDataException('招募标题')
        # 获取第一个非 None 招募标题
        try:
            return next(filter(lambda x: x is not None, gacha_name))
        except StopIteration:
            raise NoDataException('招募标题')
    
    # 获取招募默认服务器
    @property
    def server(self) -> Literal['jp', 'en', 'tw', 'cn', 'kr']:
        '''获取招募默认服务器

        返回:
            Literal[&#39;jp&#39;, &#39;en&#39;, &#39;tw&#39;, &#39;cn&#39;, &#39;kr&#39;]: 歌曲所在服务器
        '''
        info = self.__info
        # 获取 publishedAt 数据
        if (published_at := info.get('publishedAt', None)) is None:
            raise NoDataException('招募起始时间')
        # 根据 publishedAt 数据判断服务器
        if published_at[0] is not None: return 'jp'
        elif published_at[1] is not None: return 'en'
        elif published_at[2] is not None: return 'tw'
        elif published_at[3] is not None: return 'cn'
        elif published_at[4] is not None: return 'kr'
        else:
            raise NoDataException('招募所在服务器')
    
    # 获取招募信息
    def get_info(self) -> Dict[str, Any]:
        '''获取招募信息

        返回:
            Dict[str, Any]: 招募详细信息
        '''
        try:
            response = Api(
                API['gacha']['info'].format(id=self.id)
            ).get()
        except HTTPStatusError as exception:
            if exception.response.status_code == 404:
                raise GachaNotExistError(self.id)
            else:
                raise exception
        
        self.__info = dict(response.json())
        return self.__info
    
    # 异步获取招募信息
    async def get_info_async(self) -> Dict[str, Any]:
        '''获取招募信息

        返回:
            Dict[str, Any]: 招募详细信息
        '''
        try:
            response = await Api(
                API['gacha']['info'].format(id=self.id)
            ).aget()
        except ClientResponseError as exception:
            if exception.status == 404:
                raise GachaNotExistError(self.id)
            else:
                raise exception
        
        self.__info = dict(await response.json())
        return self.__info
    
    # 获取缓存信息
    def __get_info_cache(self) -> Dict[str, Any]:
        '''获取缓存信息

        返回:
            Dict[str, Any]: 缓存信息
        '''
        if not self.__info:
            return self.get_info()
        return self.__info
    
    # 异步获取缓存信息
    async def __get_info_cache_async(self) -> Dict[str, Any]:
        '''获取缓存信息

        返回:
            Dict[str, Any]: 缓存信息
        '''
        if not self.__info:
            return await self.get_info_async()
        return self.__info
    
    # 获取招募评论
    def get_comment(
        self,
        limit: int=20,
        offset: int=0,
        order: Literal['TIME_DESC', 'TIME_ASC']='TIME_ASC'
    ) -> Dict[str, Any]:
        '''获取招募评论

        参数:
            limit (int, optional): 展示出的评论数，默认为 20
            offset (int, optional): 忽略前面的 `offset` 条评论，默认为 0
            order (Literal[&#39;TIME_DESC&#39;, &#39;TIME_ASC&#39;], optional): 排序顺序，默认时间顺序

        返回:
            Dict[str, Any]: 搜索结果
                ```python
                {
                    "result": ... # bool 是否有响应
                    "count": ... # int 搜索到的评论总数
                    "posts": ... # List[Dict[str, Any]] 列举出的评论
                }
                ```
        '''
        return get_list(
            category_name='GACHA_COMMENT',
            category_id=str(self.id),
            order=order,
            limit=limit,
            offset=offset
        )
    
    # 异步获取招募评论
    async def get_comment_async(
        self,
        limit: int=20,
        offset: int=0,
        order: Literal['TIME_DESC', 'TIME_ASC']='TIME_ASC'
    ) -> Dict[str, Any]:
        '''获取招募评论

        参数:
            limit (int, optional): 展示出的评论数，默认为 20
            offset (int, optional): 忽略前面的 `offset` 条评论，默认为 0
            order (Literal[&#39;TIME_DESC&#39;, &#39;TIME_ASC&#39;], optional): 排序顺序，默认时间顺序

        返回:
            Dict[str, Any]: 搜索结果
                ```python
                {
                    "result": ... # bool 是否有响应
                    "count": ... # int 搜索到的评论总数
                    "posts": ... # List[Dict[str, Any]] 列举出的评论
                }
                ```
        '''
        return await get_list_async(
            category_name='GACHA_COMMENT',
            category_id=str(self.id),
            order=order,
            limit=limit,
            offset=offset
        )
    
    # 获取招募缩略图图片
    def get_banner(self, server: Literal['jp', 'en', 'tw', 'cn', 'kr']) -> bytes:
        '''获取招募缩略图图片

        参数:
            server (Literal[&#39;jp&#39;, &#39;en&#39;, &#39;tw&#39;, &#39;cn&#39;, &#39;kr&#39;]): 指定服务器

        返回:
            bytes: 招募缩略图图片字节数据 `bytes`
        '''
        # 获取招募数据包名称
        info = self.__get_info_cache()
        if (banner_asset_bundle_name := info.get('bannerAssetBundleName', None)) is None:
            raise ValueError('无法获取招募数据包名称。')
        # 判断服务器
        if (start_at := info.get('startAt', None)) is None:
            raise ValueError('无法获取招募起始时间。')
        SERVERS = ['jp', 'en', 'tw', 'cn', 'kr']
        index = SERVERS.index(server)
        if start_at[index] is None:
            raise ServerNotAvailableError(f'招募 {self.name}', server)
        return Assets(
            ASSETS['homebanner']['get'].format(
                banner_asset_bundle_name=banner_asset_bundle_name
            ), server
        ).get()
    
    # 异步获取招募缩略图图片
    async def get_banner_async(self, server: Literal['jp', 'en', 'tw', 'cn', 'kr']) -> bytes:
        '''获取招募缩略图图片

        参数:
            server (Literal[&#39;jp&#39;, &#39;en&#39;, &#39;tw&#39;, &#39;cn&#39;, &#39;kr&#39;]): 指定服务器

        返回:
            bytes: 招募缩略图图片字节数据 `bytes`
        '''
        # 获取招募数据包名称
        info = await self.__get_info_cache_async()
        if (banner_asset_bundle_name := info.get('bannerAssetBundleName', None)) is None:
            raise ValueError('无法获取招募数据包名称。')
        # 判断服务器
        if (start_at := info.get('startAt', None)) is None:
            raise ValueError('无法获取招募起始时间。')
        SERVERS = ['jp', 'en', 'tw', 'cn', 'kr']
        index = SERVERS.index(server)
        if start_at[index] is None:
            raise ServerNotAvailableError(f'招募 {self.name}', server)
        return await Assets(
            ASSETS['homebanner']['get'].format(
                banner_asset_bundle_name=banner_asset_bundle_name
            ), server
        ).aget()
    
    # 获取招募 pickup 图像
    def get_pickups(self, server: Literal['jp', 'en', 'tw', 'cn', 'kr']) -> List[bytes]:
        '''获取招募 pickup 图像

        参数:
            server (Literal[&#39;jp&#39;, &#39;en&#39;, &#39;tw&#39;, &#39;cn&#39;, &#39;kr&#39;]): 服务器

        返回:
            List[bytes]: pickup 图像字节数据 `bytes` 列表
        '''
        PICKUPS = ['pickup1', 'pickup2', 'pickup']
        # 遍历尝试获取
        pickup_list: List[bytes] = []
        for pickup in PICKUPS:
            try:
                pickup_list.append(
                    Assets(
                        ASSETS['gacha']['screen'].format(
                            id=self.id, asset_name=pickup
                        ), server
                    ).get()
                )
            except:
                continue
        if len(pickup_list) <= 0:
            # 没有获取到任何 pickup 图像
            raise AssetsNotExistError('招募 pickup 图像')
        return pickup_list
    
    # 异步获取招募 pickup 图像
    async def get_pickups_async(self, server: Literal['jp', 'en', 'tw', 'cn', 'kr']) -> List[bytes]:
        '''获取招募 pickup 图像

        参数:
            server (Literal[&#39;jp&#39;, &#39;en&#39;, &#39;tw&#39;, &#39;cn&#39;, &#39;kr&#39;]): 服务器

        返回:
            List[bytes]: pickup 图像字节数据 `bytes` 列表
        '''
        PICKUPS = ['pickup1', 'pickup2', 'pickup']
        # 遍历尝试获取
        pickup_list: List[bytes] = []
        tasks = [Assets(ASSETS['gacha']['screen'].format(id=self.id, asset_name=pickup), server).aget() for pickup in PICKUPS]
        for task in asyncio.as_completed(tasks):
            try:
                pickup_list.append(await task)
            except:
                continue
        if len(pickup_list) <= 0:
            # 没有获取到任何 pickup 图像
            raise AssetsNotExistError('招募 pickup 图像')
        return pickup_list
    
    # 获取招募 logo 图像
    def get_logo(self, server: Literal['jp', 'en', 'tw', 'cn', 'kr']) -> bytes:
        '''获取招募 logo 图像

        参数:
            server (Literal[&#39;jp&#39;, &#39;en&#39;, &#39;tw&#39;, &#39;cn&#39;, &#39;kr&#39;]): 服务器

        返回:
            bytes: logo 图像字节数据 `bytes`
        '''
        try:
            return Assets(
                ASSETS['gacha']['screen'].format(
                    id=self.id, asset_name='logo'
                ), server
            ).get()
        except:
            raise AssetsNotExistError('招募 logo 图像')
    
    # 异步获取招募 logo 图像
    async def get_logo_async(self, server: Literal['jp', 'en', 'tw', 'cn', 'kr']) -> bytes:
        '''获取招募 logo 图像

        参数:
            server (Literal[&#39;jp&#39;, &#39;en&#39;, &#39;tw&#39;, &#39;cn&#39;, &#39;kr&#39;]): 服务器

        返回:
            bytes: logo 图像字节数据 `bytes`
        '''
        try:
            return await Assets(
                ASSETS['gacha']['screen'].format(
                    id=self.id, asset_name='logo'
                ), server
            ).aget()
        except:
            raise AssetsNotExistError('招募 logo 图像')



'''`bestdori`

Bestdori 的各种 API 调用整合，另外附带部分功能'''

from .user import User, Me

from .post import Post
from .charts import Chart

from .characters import Character
from .cards import Card
from .costumes import Costume
from .events import Event
from .eventarchives import EventArchive
from .gacha import Gacha
from .songs import Song
from .logincampaigns import LoginCampaign
from .miracleticket import MiracleTicketExchange
from .comics import Comic
from .missions import Mission

from .utils.utils import (
    API,
    RES,
    ASSETS
)
from .utils.network import (
    Api,
    Res,
    Assets
)
from .exceptions import (
    RequestInvalidError,
    LoginRequiredError,
    CredentialsInvalidError,
    UserInvalidError,
    PostHasNoChartError,
    PostHasNoSongError,
    SongNotExistError,
    DiffNotExistError,
    EventHasNoStampError,
    ServerNotAvailableError,
    AssetsNotExistError,
    CharacterNotExistError,
    CardNotExistError,
    CostumeNotExistError,
    EventNotExistError,
    GachaNotExistError,
    SongNotExistError,
    LoginCampaignNotExistError,
    MiracleTicketExchangeNotExistError,
    ComicNotExistError,
    MissionNotExistError
)

from . import (
    user,
    post,
    charts,
    characters,
    cards,
    costumes,
    events,
    eventarchives,
    gacha,
    songs,
    songmeta,
    logincampaigns,
    miracleticket,
    comics,
    missions,
    models
)

from . import settings as settings

__all__ = [
    'User',
    'Me',
    'Post',
    'Chart',
    'Character',
    'Card',
    'Costume',
    'Event',
    'EventArchive',
    'Gacha',
    'Song',
    'LoginCampaign',
    'MiracleTicketExchange',
    'Comic',
    'Mission',
    'API',
    'RES',
    'ASSETS',
    'Api',
    'Res',
    'Assets',
    'RequestInvalidError',
    'LoginRequiredError',
    'CredentialsInvalidError',
    'UserInvalidError',
    'PostHasNoChartError',
    'PostHasNoSongError',
    'SongNotExistError',
    'DiffNotExistError',
    'EventHasNoStampError',
    'ServerNotAvailableError',
    'AssetsNotExistError',
    'CharacterNotExistError',
    'CardNotExistError',
    'CostumeNotExistError',
    'EventNotExistError',
    'GachaNotExistError',
    'SongNotExistError',
    'LoginCampaignNotExistError',
    'MiracleTicketExchangeNotExistError',
    'ComicNotExistError',
    'MissionNotExistError',
    'user',
    'post',
    'charts',
    'characters',
    'cards',
    'costumes',
    'events',
    'eventarchives',
    'gacha',
    'songs',
    'songmeta',
    'logincampaigns',
    'miracleticket',
    'comics',
    'missions',
    'models',
    'settings'
]


'''`bestdori.utils.utils`

工具库模块'''

API = {
    'user': {
        'info': 'user',
        'login': 'user/login',
        'me': 'user/me'
    },
    'post': {
        'basic': 'post/basic',
        'details': 'post/details',
        'list': 'post/list',
        'tag': 'post/tag',
        'post': 'post',
        'find': 'post/find',
        'like': 'post/like'
    },
    'charts': {
        'info': 'charts/{id}/{diff}.json'
    },
    'characters': {
        'info': 'characters/{id}.json',
        'all': 'characters/all.{index}.json'
    },
    'cards': {
        'info': 'cards/{id}.json',
        'all': 'cards/all.{index}.json'
    },
    'costumes': {
        'info': 'costumes/{id}.json',
        'all': 'costumes/all.{index}.json'
    },
    'events': {
        'info': 'events/{id}.json',
        'all': 'events/all.{index}.json',
        'top': 'eventtop/data'
    },
    'festival':{
        'stages': 'festival/stages/{id}.json',
        'rotation_musics': 'festival/rotationMusics/{id}.json',
    },
    'gacha': {
        'info': 'gacha/{id}.json',
        'all': 'gacha/all.{index}.json'
    },
    'songs': {
        'info': 'songs/{id}.json',
        'all': 'songs/all.{index}.json'
    },
    'loginCampaigns': {
        'info': 'loginCampaigns/{id}.json',
        'all': 'loginCampaigns/all.{index}.json'
    },
    'bands': {
        'all': 'bands/all.{index}.json',
        'main': 'bands/main.{index}.json'
    },
    'upload': {
        'file': 'upload/file/{hash}',
        'prepare': 'upload/prepare',
        'upload': 'upload',
        'status': 'upload/status/{hash}'
    },
    'misc': {
        'llsif': 'misc/llsif.{index}.json'
    },
    'player': {
        'get': 'player/{server}/{id}'
    },
    'all': {
        'skills': 'skills/all.{index}.json',
        'stamps': 'stamps/all.{index}.json',
        'degrees': 'degrees/all.{index}.json',
        'meta': 'songs/meta/all.{index}.json',
        'archives': 'archives/all.{index}.json',
        'miracleTicketExchanges': 'miracleTicketExchanges/all.{index}.json',
        'comics': 'comics/all.{index}.json',
    }
}
'''Bestdori API 集合'''

ASSETS = {
    'characters': {
        'character_kv_image': 'ui/character_kv_image/{id:>03d}_rip/image.png',
        'resourceset': 'characters/resourceset/{resource_set_name}_rip/{name}_{type}.png',
        'livesd': 'characters/livesd/{sd_resource_name}_rip/sdchara.png'
    },
    'event': {
        'banner': 'event/{asset_bundle_name}/images_rip/banner.png',
        'logo': 'event/{asset_bundle_name}/images_rip/logo.png',
        'topscreen': 'event/{asset_bundle_name}/topscreen_rip/{type}_eventtop.png',
        'loginbouns': 'event/loginbonus/{asset_bundle_name}_rip/background.png'
    },
    'songs': {
        'musicjacket': 'musicjacket/musicjacket{index:>02d}_rip/assets-star-forassetbundle-startapp-musicjacket-musicjacket{index:>02d}-{jacket_image}-jacket.png',
        'sound': 'sound/bgm{id:>03d}_rip/bgm{id:>03d}.mp3',
        'musicscore': ''
    },
    'thumb': {
        'chara': 'thumb/chara/card{id:>05d}_rip/{resource_set_name}_{type}.png',
        'degree': 'thumb/degree_rip/{degree_name}.png',
        'costume': 'thumb/costume/group{id}_rip/{asset_bundle_name}.png',
    },
    'stamp': {
        'get': 'stamp/01_rip/{image_name}.png'
    },
    'homebanner': {
        'get': 'homebanner_rip/{banner_asset_bundle_name}.png'
    },
    'gacha': {
        'screen': 'gacha/screen/gacha{id}_rip/{asset_name}.png'
    },
    'comic': {
        'comic': 'comic/comic_{type}/{asset_bundle_name}_rip/{asset_bundle_name}.png',
        'thumbnail': 'comic/comic_{type}_thumbnail/{asset_bundle_name}_rip/{asset_bundle_name}.png'
    },
    'missions': {
        'info': 'missions/{id}.json',
        'all': 'missions/all.{index}.json'
    },
    'band': {
        'logo': 'band/logo/{id:>03d}_rip/{type}.png'
    },
    'live2d': {
        'buildData': 'live2d/chara/{asset_bundle_name}_rip/buildData.asset'
    }
}
'''Bestdori 数据包集合'''

RES = {
    'icon': {
        'svg': 'icon/{name}.svg',
        'png': 'icon/{name}.png'
    },
    'image': {
        'png': 'image/{name}.png'
    },
    'emoji': {
        'png': 'emoji/{name}.png'
    }
}
'''Bestdori 资源集合'''


'''`bestdori.ayachan.utils.network`

向 ayachan 发送请求相关模块'''
from json import dumps
from io import BufferedReader
from typing import Any, Dict, Tuple, Union, Literal, Optional, cast

from httpx import HTTPStatusError
from aiohttp import ClientResponseError
from aiohttp import ClientSession, ClientResponse
from httpx import Client, Request, Response, AsyncClient

from . import settings
from bestdori.exceptions import AssetsNotExistError
from bestdori.settings import AsyncClient as ClientSetting

# 向 ayachan 发送 API 请求类
class Api:
    '''向 ayachan 发送 API 请求类

    参数:
        api (str): 请求的 API 地址
    '''
    api: str
    '''请求的 API 地址'''
    _proxies: Optional[Union[Dict[str, str], str]]=None
    '''代理服务器'''
    # 初始化
    def __init__(
        self,
        api: str
    ) -> None:
        '''初始化'''
        self.api = api
        if isinstance(settings.proxy, str):
            self._proxies = {'http://': settings.proxy, 'https://': settings.proxy}
        elif isinstance(settings.proxy, dict):
            self._proxies = settings.proxy
        return
    
    # 处理 url
    @property
    def _url(self) -> str:
        # 处理接收到的 API
        _url = self.api
        if _url.startswith('http://') or _url.startswith('https://'):
            return _url
        else:
            _url = 'https://api.ayachan.fun/' + self.api
            return _url
    
    # 请求发送
    def _request(
        self,
        method: Literal['get', 'post'],
        *,
        params: Optional[Dict[str, Any]]=None,
        data: Optional[Any]=None,
        files: Optional[Dict[str, Tuple[str, BufferedReader, Optional[str]]]]=None,
        json: Optional[Any]=None
    ) -> Response:
        '''请求发送

        参数:
            method (Literal[&#39;get&#39;, &#39;post&#39;]): API 调用方法
            params (Optional[Dict[str, Any]], optional): 调用参数
            data (Optional[Any], optional): 调用参数，将以 `json` 字符串形式发送
            files (Optional[Dict[str, Tuple[str, BufferedReader, Optional[str]]]], optional): 发送文件参数
            json (Optional[Any], optional): 调用参数，将以 `json` 字符串形式发送

        返回:
            Response: 收到的响应
        '''
        # 构建一个请求体
        request = Request(
            method,
            self._url,
            params=params,
            data=(
                cast(dict, dumps(data)) if data is not None and 'sonolus' not in self.api
                else data
            ),
            files=files,
            json=json,
            headers=(
                {
                    'Content-Type': 'multipart/form-data; boundary=----WebKitFormBoundaryxjpDgorEXUAUIbBN'
                } if 'sonolus' in self.api and method == 'post' else None
            )
        )
        
        # 发送请求并获取响应
        with Client(proxies=cast(dict, self._proxies), timeout=settings.timeout, trust_env=False) as client:
            response = client.send(request)
        
        # 处理接收到的响应
        response.raise_for_status()
        return response
    
    # 异步请求发送
    async def _request_async(
        self,
        method: Literal['get', 'post'],
        *,
        params: Optional[Dict[str, Any]]=None,
        data: Optional[Any]=None,
        files: Optional[Dict[str, Tuple[str, BufferedReader, Optional[str]]]]=None,
        json: Optional[Any]=None
    ) -> Union[Response, ClientResponse]:
        '''请求发送

        参数:
            method (Literal[&#39;get&#39;, &#39;post&#39;]): API 调用方法
            params (Optional[Dict[str, Any]], optional): 调用参数
            data (Optional[Any], optional): 调用参数，将以 `json` 字符串形式发送
            files (Optional[Dict[str, Tuple[str, BufferedReader, Optional[str]]]], optional): 发送文件参数
            json (Optional[Any], optional): 调用参数，将以 `json` 字符串形式发送

        返回:
            Response: 收到的响应
        '''
        if settings.async_client == ClientSetting.HTTPX:
            # 构建一个请求体
            request = Request(
                method,
                self._url,
                params=params,
                data=(
                    cast(dict, dumps(data)) if data is not None and 'sonolus' not in self.api
                    else data
                ),
                files=files,
                json=json,
                headers=(
                    {
                        'Content-Type': 'multipart/form-data; boundary=----WebKitFormBoundaryxjpDgorEXUAUIbBN'
                    } if 'sonolus' in self.api and method == 'post' else None
                )
            )
            
            # 发送请求并获取响应
            async with AsyncClient(proxies=cast(dict, self._proxies), timeout=settings.timeout, trust_env=False) as client:
                response = await client.send(request)
            
                # 处理接收到的响应
                response.raise_for_status()
        else:
            async with ClientSession() as session:
                async with session.request(
                    method,
                    self._url,
                    params=params,
                    data=(
                        cast(dict, dumps(data)) if data is not None and 'sonolus' not in self.api
                        else data
                    ),
                    files=files,
                    json=json,
                    headers=(
                        {
                            'Content-Type': 'multipart/form-data; boundary=----WebKitFormBoundaryxjpDgorEXUAUIbBN'
                        } if 'sonolus' in self.api and method == 'post' else None
                    )
                ) as response:
                    response.raise_for_status()
        
        return response

    def get(
        self,
        params: Optional[Dict[str, Any]]=None
    ) -> Response:
        '''发送 GET 请求

        参数:
            params (Optional[Dict[str, Any]], optional): 调用参数

        返回:
            Response: 收到的响应
        '''
        return self._request('get', params=params)
    
    async def aget(
        self,
        params: Optional[Dict[str, Any]]=None
    ) -> Union[Response, ClientResponse]:
        '''发送 GET 请求

        参数:
            params (Optional[Dict[str, Any]], optional): 调用参数

        返回:
            Response: 收到的响应
        '''
        return await self._request_async('get', params=params)
    
    def post(
        self,
        data: Optional[Any]=None,
        files: Optional[Dict[str, Tuple[str, BufferedReader, Optional[str]]]]=None,
        json: Optional[Any]=None
    ) -> Response:
        '''发送 POST 请求

        参数:
            data (Optional[Any], optional): 调用参数，将以 `json` 字符串形式发送
            files (Optional[Dict[str, Tuple[str, BufferedReader, Optional[str]]]], optional): 发送文件参数
            json (Optional[Any], optional): 调用参数，将以 `json` 字符串形式发送

        返回:
            Response: 收到的响应
        '''
        return self._request('post', data=data, files=files, json=json)
    
    async def apost(
        self,
        data: Optional[Any]=None,
        files: Optional[Dict[str, Tuple[str, BufferedReader, Optional[str]]]]=None,
        json: Optional[Any]=None
    ) -> Union[Response, ClientResponse]:
        '''发送 POST 请求

        参数:
            data (Optional[Any], optional): 调用参数，将以 `json` 字符串形式发送
            files (Optional[Dict[str, Tuple[str, BufferedReader, Optional[str]]]], optional): 发送文件参数
            json (Optional[Any], optional): 调用参数，将以 `json` 字符串形式发送

        返回:
            Response: 收到的响应
        '''
        return await self._request_async('post', data=data, files=files, json=json)

# 获取 ayachan 资源数据
class Assets:
    '''获取 Bestdori 资源数据

    参数:
        url (str): 请求的资源地址
        proxy (Optional[Union[Dict[str, str], str]]): 代理服务器
    '''
    url: str
    '''请求的资源地址'''
    _proxies: Optional[Union[Dict[str, str], str]]=None
    '''代理服务器'''
    # 初始化
    def __init__(
        self,
        url: str
    ) -> None:
        '''获取 Bestdori 资源数据

        参数:
            url (str): 请求的资源地址
            proxy (Optional[Union[Dict[str, str], str]]): 代理服务器
        '''
        self.url = url
        if isinstance(settings.proxy, str):
            self._proxies = {'http://': settings.proxy, 'https://': settings.proxy}
        elif isinstance(settings.proxy, dict):
            self._proxies = settings.proxy
        return
    
    # 获取资源连接
    def get_url(self) -> str:
        '''获取资源连接

        返回:
            str: 获取的资源连接 `str`
        '''
        # 处理接收到的 URL
        if self.url.startswith('http://') or self.url.startswith('https://'):
            return self.url
        else:
            return 'https://api.ayachan.fun/' + self.url
    
    # 获取资源
    def get(self) -> bytes:
        '''获取资源

        返回:
            bytes: 获取的资源字节数据 `bytes`
        '''
        # 构建一个请求体
        request = Request('get', self.get_url())
        
        # 发送请求并获取响应
        with Client(proxies=cast(dict, self._proxies), timeout=settings.timeout, trust_env=False) as client:
            response = client.send(request)
        
        try:
            response.raise_for_status()
        except HTTPStatusError as exception:
            if exception.response.status_code == 404:
                raise AssetsNotExistError(self.url)
            else:
                raise exception
        # 检测响应资源是否存在
        content_type = response.headers.get('content-type', None)
        if content_type is None or content_type == 'text/html':
            raise AssetsNotExistError(self.url)
        return response.content
    
    # 异步获取资源
    async def aget(self) -> bytes:
        '''获取资源

        返回:
            bytes: 获取的资源字节数据 `bytes`
        '''
        if settings.async_client == ClientSetting.HTTPX:
            # 构建一个请求体
            request = Request('get', self.url)
            
            # 发送请求并获取响应
            async with AsyncClient(proxies=cast(dict, self._proxies), timeout=settings.timeout, trust_env=False) as client:
                response = await client.send(request)
            
            try:
                response.raise_for_status()
            except HTTPStatusError as exception:
                if exception.response.status_code == 404:
                    raise AssetsNotExistError(self.url)
                else:
                    raise exception
        else:
            # 发送请求并获取响应
            async with ClientSession() as session:
                response = await session.request(
                    'get',
                    self.get_url(),
                    proxy=self._proxies,
                )
                try:
                    response.raise_for_status()
                except ClientResponseError as exception:
                    if exception.status == 404:
                        raise AssetsNotExistError(self.url)
                    else:
                        raise exception
        
        # 检测响应资源是否存在
        content_type = response.headers.get('content-type', None)
        if content_type is None or content_type == 'text/html':
            raise AssetsNotExistError(self.url)
        
        if isinstance(response, Response):
            return response.content
        return await response.read()



'''`bestdori.settings` 设置项'''

from enum import Enum
from typing import TYPE_CHECKING, Dict, Union, Optional

from http.cookies import SimpleCookie

from .exceptions import NoCookiesError

if TYPE_CHECKING:
    from .user import Me

class AsyncClient(str, Enum):
    '''异步客户端'''
    HTTPX = 'httpx'
    AIO_HTTP = 'aiohttp'

proxy: Optional[Union[Dict[str, str], str]] = None
'''代理服务器

若想要使用代理，则必须设置该选项，
因为 `bestdori` 将会在内部默认无视系统环境下的代理设置
'''
async_client: AsyncClient = AsyncClient.HTTPX
'''异步客户端

默认为 `httpx`，可选 `aiohttp`
'''
cookies: Optional[SimpleCookie] = None
'''Cookies

不建议直接设置，推荐通过 `Settings.set_user` 方法，
传入传入用户名与密码设置
'''
timeout: int = 10
'''超时时间'''
_username: Optional[str] = None
'''用户名'''
_password: Optional[str] = None
'''密码'''
_me: Optional['Me'] = None
'''`Me` 对象'''

def set_user(username: str, password: str) -> None:
    '''设置用户名与密码'''
    global _username, _password
    
    _username = username
    _password = password

def _cookies() -> SimpleCookie:
    '''获取 Cookies'''
    global cookies, _username, _password, _me
    
    if cookies:
        return cookies
    if _me is not None:
        return _me.cookies
    raise NoCookiesError()



'''`bestdori.eventarchives`

BanG Dream! 活动数据相关操作'''
from typing import Any, Dict, List, Literal

from httpx import Response

from .utils.utils import API
from .utils.network import Api
from .post import get_list, get_list_async
from .exceptions import (
    EventNotExistError
)

# 获取总活动数据信息
def get_all(index: Literal[5]=5) -> Dict[str, Dict[str, Any]]:
    '''获取总活动信息

    参数:
        index (Literal[5], optional): 指定获取哪种 `all.json`
            `5`: 获取所有已有活动数据的简洁信息 `all.5.json`

    返回:
        Dict[str, Dict[str, Any]]: 获取到的总活动信息
    '''
    return Api(API['all']['archives'].format(index=index)).get().json()

# 异步获取总活动数据信息
async def get_all_async(index: Literal[5]=5) -> Dict[str, Dict[str, Any]]:
    '''获取总活动信息

    参数:
        index (Literal[5], optional): 指定获取哪种 `all.json`
            `5`: 获取所有已有活动数据的简洁信息 `all.5.json`

    返回:
        Dict[str, Dict[str, Any]]: 获取到的总活动信息
    '''
    response = await Api(API['all']['archives'].format(index=index)).aget()
    if isinstance(response, Response):
        return response.json()
    else:
        return await response.json()

# 活动数据类
class EventArchive:
    '''活动数据类

    参数:
        id (int): 活动 ID
    '''
    # 初始化
    def __init__(self, id: int) -> None:
        '''活动数据类

        参数:
            id (int): 活动 ID
        '''
        self.id: int = id
        '''活动 ID'''
        self.__info: Dict[str, Any] = {}
        '''活动信息'''
        return
    
    # 获取活动数据信息
    def get_info(self) -> Dict[str, Any]:
        '''获取活动数据信息

        返回:
            Dict[str, Any]: 活动数据信息
        '''
        _all = get_all()
        if str(self.id) not in _all:
            raise EventNotExistError(self.id)
        self.__info = _all[str(self.id)]
        
        return self.__info
    
    # 异步获取活动数据信息
    async def get_info_async(self) -> Dict[str, Any]:
        '''获取活动数据信息

        返回:
            Dict[str, Any]: 活动数据信息
        '''
        _all = await get_all_async()
        if str(self.id) not in _all:
            raise EventNotExistError(self.id)
        self.__info = _all[str(self.id)]
        
        return self.__info
    
    # 获取排名分数线
    def get_top(
        self,
        server: Literal[0, 1, 2, 3, 4],
        mid: Literal['0']='0',
        latest: Literal['1']='1'
    ) -> Dict[str, List[Dict[str, Any]]]:
        '''获取排名分数线

        参数:
            server (Literal[0, 1, 2, 3, 4]): 指定服务器
                `0`: 日服
                `1`: 英服
                `2`: 台服
                `3`: 国服
                `4`: 韩服
            mid (Literal[&#39;0&#39;], optional): 指定是否为中间分数线，默认为 `0`
            latest (Literal[&#39;1&#39;], optional): 指定是否为最终分数线，默认为 `1`

        返回:
            Dict[str, List[Dict[str, Any]]]: 排名分数线数据
        '''
        return Api(API['events']['top']).get(
            params={
                'server': server,
                'event': self.id,
                'mid': mid,
                'latest': latest
            }
        ).json()
    
    # 异步获取排名分数线
    async def get_top_async(
        self,
        server: Literal[0, 1, 2, 3, 4],
        mid: Literal['0']='0',
        latest: Literal['1']='1'
    ) -> Dict[str, List[Dict[str, Any]]]:
        '''获取排名分数线

        参数:
            server (Literal[0, 1, 2, 3, 4]): 指定服务器
                `0`: 日服
                `1`: 英服
                `2`: 台服
                `3`: 国服
                `4`: 韩服
            mid (Literal[&#39;0&#39;], optional): 指定是否为中间分数线，默认为 `0`
            latest (Literal[&#39;1&#39;], optional): 指定是否为最终分数线，默认为 `1`

        返回:
            Dict[str, List[Dict[str, Any]]]: 排名分数线数据
        '''
        response = await Api(API['events']['top']).aget(
            params={
                'server': server,
                'event': self.id,
                'mid': mid,
                'latest': latest
            }
        )
        if isinstance(response, Response):
            return response.json()
        else:
            return await response.json()

    # 获取活动数据评论
    def get_comment(
        self,
        limit: int=20,
        offset: int=0,
        order: Literal['TIME_DESC', 'TIME_ASC']='TIME_ASC'
    ) -> Dict[str, Any]:
        '''获取动数据评论

        参数:
            limit (int, optional): 展示出的评论数，默认为 20
            offset (int, optional): 忽略前面的 `offset` 条评论，默认为 0
            order (Literal[&#39;TIME_DESC&#39;, &#39;TIME_ASC&#39;], optional): 排序顺序，默认时间顺序

        返回:
            Dict[str, Any]: 搜索结果
                ```python
                {
                    "result": ... # bool 是否有响应
                    "count": ... # int 搜索到的评论总数
                    "posts": ... # List[Dict[str, Any]] 列举出的评论
                }
                ```
        '''
        return get_list(
            category_id=str(self.id),
            category_name='EVENTARCHIVE_COMMENT',
            limit=limit,
            offset=offset,
            order=order
        )
    
    # 异步获取活动数据评论
    async def get_comment_async(
        self,
        limit: int=20,
        offset: int=0,
        order: Literal['TIME_DESC', 'TIME_ASC']='TIME_ASC'
    ) -> Dict[str, Any]:
        '''获取动数据评论

        参数:
            limit (int, optional): 展示出的评论数，默认为 20
            offset (int, optional): 忽略前面的 `offset` 条评论，默认为 0
            order (Literal[&#39;TIME_DESC&#39;, &#39;TIME_ASC&#39;], optional): 排序顺序，默认时间顺序

        返回:
            Dict[str, Any]: 搜索结果
                ```python
                {
                    "result": ... # bool 是否有响应
                    "count": ... # int 搜索到的评论总数
                    "posts": ... # List[Dict[str, Any]] 列举出的评论
                }
                ```
        '''
        return await get_list_async(
            category_id=str(self.id),
            category_name='EVENTARCHIVE_COMMENT',
            limit=limit,
            offset=offset,
            order=order
        )
